/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.  
 */


/**
 * @author Neeraj Joshi <jneeraj@us.ibm.com>
 *
 *  This class provides methods to map java types to SPL types and vice
 *  versa
 */
package org.apache.imperius.javaspl;


import java.util.logging.Logger;

import org.apache.imperius.spl.core.TypeConstants;
import org.apache.imperius.spl.parser.exceptions.TypeMismatchException;
import org.apache.imperius.util.SPLLogger;



public class JavaSPLTypeConstants
{

    private static Logger logger = SPLLogger.getSPLLogger().getLogger();
    private static final String sourceClass="JavaSPLTypeConstants";
    
    public static boolean getIsArray(String type)
    {
    	if(type.startsWith("["))
    	{
    		return true;
    	}
    	return false;
    	
    }

    public static Class convertInternalTypeToJavaType(int type, boolean isArray, 
    		String referenceName) throws ClassNotFoundException
    {
    	Class javaType = null;
    	if(type == TypeConstants.byteType)
    	{
    		if(isArray)
    		{
    			javaType = Class.forName("[B");
    		}
    		else
    		{
    			javaType = Byte.TYPE;
    		}
    	}
    	else if(type == TypeConstants.shortType)
    	{
    		if(isArray)
    		{
    			javaType = Class.forName("[S");
    		}
    		else
    		{
    			javaType = Short.TYPE;
    		}
    	}
    	else if(type == TypeConstants.intType)
    	{
    		if(isArray)
    		{
    			javaType = Class.forName("[I");
    		}
    		else
    		{
    			javaType = Integer.TYPE;
    		}
    	}
    	else if(type == TypeConstants.longType)
    	{
    		if(isArray)
    		{
    			javaType = Class.forName("[J");
    		}
    		else
    		{
    			javaType = Long.TYPE;
    		}
    	}
    	else if(type == TypeConstants.floatType)
    	{
    		if(isArray)
    		{
    			javaType = Class.forName("[F");
    		}
    		else
    		{
    			javaType = Float.TYPE;
    		}
    	}
    	else if(type == TypeConstants.doubleType)
    	{
    		if(isArray)
    		{
    			javaType = Class.forName("[D");
    		}
    		else
    		{
    			javaType = Double.TYPE;
    		}
    	}
    	else if(type == TypeConstants.booleanType)
    	{
    		if(isArray)
    		{
    			javaType = Class.forName("[Z");
    		}
    		else
    		{
    			javaType = Boolean.TYPE;
    		}
    	}
    	else if(type == TypeConstants.charType)
    	{
    		if(isArray)
    		{
    			javaType = Class.forName("[C");
    		}
    		else
    		{
    			javaType = Character.TYPE;
    		}
    	}
    	else if(type == TypeConstants.stringType)
    	{
    		if(isArray)
    		{
    			javaType = Class.forName("[Ljava.lang.String;");
    		}
    		else
    		{
    			javaType = Class.forName("java.lang.String");
    		}
    	}
    	else if(type == TypeConstants.dateTime)
    	{
    		if(isArray)
    		{
    			javaType = Class.forName("[Ljava.util.Calendar;");
    		}
    		else
    		{
    			javaType = Class.forName("Ljava.util.Calendar");
    		}
    	}
    	else if(type == TypeConstants.referenceType)
    	{
    		if(isArray)
    		{
    			javaType = Class.forName("[L" + referenceName + ";");
    		}
    		else
    		{
    			javaType = Class.forName(referenceName);
    		}
    	}

    	return javaType;
    }
    
    public static int convertJavaTypeToInternalType(String type)
    {
               
        
        int retTp = TypeConstants.INVALID;
        
        if (type.equals("byte") || type.equals("[B"))
        {
            retTp = TypeConstants.byteType;
        }
        else if (type.equals("short") || type.equals("[S"))
        {
            retTp = TypeConstants.shortType;
        }
        else if (type.equals("int") || type.equals("[I"))
        {
            retTp = TypeConstants.intType;
        }
        else if (type.equals("long") || type.equals("[J"))
        {
            retTp = TypeConstants.longType;
        }
        else if (type.equals("float") || type.equals("[F"))
        {
            retTp = TypeConstants.floatType;
        }
        else if (type.equals("double") || type.equals("[D"))
        {
            retTp = TypeConstants.doubleType;
        }
        else if (type.equals("boolean") || type.equals("[Z"))
        {
            retTp = TypeConstants.booleanType;
        }
        else if (type.equals("char") || type.equals("[C"))
        {
            retTp = TypeConstants.charType;
        }
        else if (type.equals("java.lang.String") 
        		|| type.equals("[Ljava.lang.String;")
        		|| type.equals("class java.lang.String"))
        {
            retTp = TypeConstants.stringType;
        }
        else if (type.equals("java.util.Calendar") || type.equals("[Ljava.util.Calendar;"))
       	{
        	retTp = TypeConstants.dateTime;
       	}
        else 
        {
            retTp = TypeConstants.referenceType;
            
        }
        return retTp;
        
    }
    
    
    
    public static int getTypeOfObject(Object ob) throws TypeMismatchException
    {
        logger.entering(sourceClass,Thread.currentThread().getName()+" "+ "getTypeOfObject");
        
        int currentLocaltype = -1;
        if (ob instanceof java.lang.Boolean)
        {	
            currentLocaltype = TypeConstants.booleanType;
        }    
        else if (ob instanceof java.lang.String)
        {	
            currentLocaltype = TypeConstants.stringType;
        }    
        else if (ob instanceof java.util.Calendar || ob instanceof java.util.Date)
        {	
            currentLocaltype = TypeConstants.dateTime;
        }    
        else if (ob instanceof java.lang.Byte)
        {	
            currentLocaltype = TypeConstants.byteType;
        }    
        else if (ob instanceof java.lang.Character)
        {	
            currentLocaltype = TypeConstants.charType;
        }    
        else if (ob instanceof java.lang.Short)
        {	
            currentLocaltype = TypeConstants.shortType;
        }    
        else if (ob instanceof java.lang.Integer)
        {	
            currentLocaltype = TypeConstants.intType;
        }    
        else if (ob instanceof java.lang.Long)
        {	
            currentLocaltype = TypeConstants.longType;
        }    
        else if (ob instanceof java.lang.Float)
        {	
            currentLocaltype = TypeConstants.floatType;
        }    
        else if (ob instanceof java.lang.Double)
        {	
            currentLocaltype = TypeConstants.doubleType;
        }    
        else 
        {	
            currentLocaltype = TypeConstants.referenceType;
        }    
       
           
        return currentLocaltype;
    }
	public static String getReferenceTypeName(String javaType) 
	{
		
		//extract class name and return
		String referenceType = null;
		if(convertJavaTypeToInternalType(javaType) == TypeConstants.referenceType)
		{	
			if(javaType.startsWith("["))
			{
				referenceType = javaType.substring(1);
			
			}
			else
			{
				if(javaType.startsWith("class ")) // of type class com.ibm.xxx
				{
					int ind = javaType.indexOf(' ');
					referenceType = javaType.substring(ind + 1);
				}
				else
				{
					referenceType = javaType;
				}	
			}
		}
		else
		{
			System.out.println("Only reference types have referenceType names" + javaType);
		}
		return referenceType;
	}
}
