/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.  
 */


/**
 * @author Prashant Baliga <prabalig@in.ibm.com>
 * This interface needs to be implemented by a binding provider.
 * 
 * It collects data from the operating environment and formats it
 * for use by the SPL engine
 */

package org.apache.imperius.spl.core;

import java.util.List;
import java.util.Map;

import org.apache.imperius.spl.parser.exceptions.SPLException;




public interface DataCollector
{
   /**
    * This method is applicable to Domains that support the notion of
    * associations. Associations are relations between 2 classes where
    * each object has a role name and the association itself is a class.
    * 
    * This method retrieves instances that are associated via a given 
    * association and roles to a source instance.
    * 
    * This method is used to get instances o
    * @param className
    * @param classPath
    * @param resultInstancesClassFilter
    * @param assocClass
    * @param role
    * @param resultRole
    * @param srcReference
    * @return
    * @throws SPLException
    */ 
	public List getAssociatedInstanceReferences(Object srcReference,
			String namespace, 
			String resultInstancesClassFilter, 
			String assocClass, 
			String role, 
			String resultRole) throws SPLException ;
	
	/**
	 * Returns a Map of all the symbols (Properties and Methods) of the given class
	 * The Map is of the form String -> Symbol where Symbol can be a PropertySymbol or a MethodSymbol
	 * 
	 * @param className
	 * @param qualifier
	 * @return Map of symbols 
	 * @throws SPLException
	 */
    public Map getSymbolsForClass(String className, String qualifier)
            throws SPLException;
    
    /**
     * Returns a Map of instance values for the given instance
     * The Map is of the form String (name of property or method)  -> Object (instance value)
     *  
     * @param className
     * @param qualifier
     * @param instance
     * @return
     * @throws SPLException
     */
    
    public Map getSymbolsForInstance(String className, String qualifier, Object instance)
            throws SPLException;
    
   
    /**
     * Determines whether an association between sourceClass and targetClass with
     * the specified roles exist. This method is applicable only when associations 
     * between classes are supported by the operating environment
     * @param nameSpace
     * @param className
     * @param resultClass
     * @param assocClass
     * @param role
     * @param resultRole
     * @return
     * @throws SPLException
     */   
    public boolean associationExists(String nameSpace, String sourceClassName,
            String targetClassName, String assocClass, String role,
            String resultRole) throws SPLException;
    /**
     * Get a list of Strings representing names of instances of the given class
     * in the Operating Environment
     * @param className
     * @param qualifier
     * @return
     * @throws SPLException
     */
    
    public List enumerateInstanceNames(String className, String qualifier)
	throws SPLException;
//    
//    public String getReferenceTypeName(String reference);
//    
    
    /**
     * Checks if the given object is an instance of the given class
     */
    public boolean isInstance(String className, Object instance)
    throws SPLException;
    
}