/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.  
 */


/**
 * @author Prashant Baliga <prabalig@in.ibm.com>
 *
 */


package org.apache.imperius.spl.core;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.logging.Level;
import java.util.logging.Logger;

import org.apache.imperius.spl.manager.PolicyManager;
import org.apache.imperius.spl.parser.exceptions.SPLException;
import org.apache.imperius.spl.parser.util.DataCollectorFactory;
import org.apache.imperius.util.Messages;
import org.apache.imperius.util.SPLLogger;


public class InternalClient
{
	private static Logger logger = SPLLogger.getSPLLogger().getLogger();
	public static final int POLICY_EVALUATED_SUCCESSFULLY = 0;
	public static final int POLICY_NOT_EVALUATED = 1;
	public static final int POLICY_EVALUATION_FAILED = -1;
	private PolicyManager pm = null;
	private DataCollector dc = null;
	private static final String sourceClass = "InternalClient";
	
    public InternalClient( PolicyManager policyManager) 
    {
        logger.entering(sourceClass, Thread.currentThread().getName() + " "
				+ "InternalClient");

		dc = DataCollectorFactory.getDataCollector();
		pm = policyManager;

		logger.exiting(sourceClass, Thread.currentThread().getName() + " "
				+ "InternalClient");
    }  

    public int invokePolicy(String policyName, Map instanceMap)
    {
    	logger.entering(sourceClass, Thread.currentThread().getName() + " "
				+ "InvokePolicy");
    	
    	int returnValue = 0;
        try
        {
            PolicyInfo policyInfo = pm.getPolicyInfo(policyName);
            if(policyInfo == null)
            {
                throw new SPLException(Messages
						.getString("SPL_POLICY_DOES_NOT_EXIST_EXCEPTION_MSG")
						+ " " + policyName);
            }
            
            if(instanceMap != null && instanceMap.size() > 0)
            {
            	logger.fine(Thread.currentThread().getName()
						+ " evaluating policy...");
            	
            	Integer retVal = (Integer) pm.evaluatePolicy(policyName,instanceMap);
            	
            	logger.fine(Thread.currentThread().getName()
						+ " policy evaluation completed successfully");
                
                if (retVal.intValue() == POLICY_EVALUATION_FAILED)
                {
                    returnValue++;
                }
                logger.exiting(sourceClass, Thread.currentThread().getName()
						+ " " + "invokePolicy"); 
            }
            return 0;
        }
        catch(SPLException e)
        {
            logger.log(Level.SEVERE, e.getMessage());
        }
        logger.exiting(sourceClass, Thread.currentThread().getName()
				+ " " + "invokePolicy"); 
        return -1;
    }
   
    public int invokePolicy(String policyName, List paramList, boolean boo)
    {
        logger.entering(sourceClass, Thread.currentThread().getName() + " "
				+ "InvokePolicy");
        
        int returnValue = 0;
		List objectrefList = paramList;
        try
        {
            if (objectrefList == null)
            {
                Map objectMap = new HashMap();
				PolicyInfo policyInfo = pm.getPolicyInfo(policyName);
				Map classQuaifierMap = policyInfo.getClassQualifierMap();
				Iterator classit = classQuaifierMap.keySet().iterator();

                if (classQuaifierMap.size() == 0 || classQuaifierMap.size() > 1)
                {
                    logger.fine("number of classes in policy should be one");
                }
                while(classit.hasNext())
                {
                    String className = (String) classit.next();
					String nameSpace = (String) classQuaifierMap.get(className);
					List instanceList = dc.enumerateInstanceNames(className,	nameSpace);
					Object instanceObj = null;
					Iterator it = instanceList.iterator();
                    while(it.hasNext())
                    {   
                    	instanceObj = (Object) it.next();
						String objectName = className;
						InstanceInfo iI = new InstanceInfo(objectName, instanceObj);
						List instanceInfoList = new ArrayList();
						instanceInfoList.add(iI);
						objectMap.put(objectName, instanceInfoList);
						Integer retVal = (Integer) pm.evaluatePolicy(
								policyName, objectMap);

                        logger.fine(Thread.currentThread().getName()+" return value is = "+retVal);
                        
                        logger.exiting(sourceClass,Thread.currentThread().getName()+" "+"invokePolicy"); 
                        
                        if (retVal.intValue() ==POLICY_EVALUATION_FAILED)
                        {
                            logger.severe("POLICY_EVALUATION_FAILED "+policyName+" "+instanceObj.toString()+" "+retVal);
                            returnValue++;
                        }
                        else
                        {
                            logger.severe("POLICY_EVALUATION_SUCCESSFUL "+policyName+" "+instanceObj.toString()+" "+retVal);
                        }
			            if(logger.isLoggable(Level.FINE))
                        logger.fine(Thread.currentThread().getName()+" Internal client return value="+returnValue);
                        objectMap.remove(instanceInfoList);
                    }
                    logger.fine("internal client finished iterating over all instances ");
                    break;
                }
            }
            else
            {
                Map objectMap=new HashMap();
                PolicyInfo policyInfo=pm.getPolicyInfo(policyName); 
                Map classQuaifierMap=policyInfo.getClassQualifierMap();
                Iterator classit=classQuaifierMap.keySet().iterator();

                logger.fine("number of classes in policy = "+classQuaifierMap.size());
                
                if(classQuaifierMap.size()==0 || classQuaifierMap.size()>1)
                {
                    logger.fine("number of classes in policy should be one");
                }
                while(classit.hasNext()){
                    String className=(String)classit.next();
                    List copList=objectrefList;
                    Iterator it=copList.iterator();
                    while(it.hasNext())
                    {   
                        Object copI=(Object)it.next();
                
                        logger.fine("instance objectpath= "+copI.toString());
                        
                        String objectName=className;
                        objectMap.put(objectName, copI);
                        Integer retVal=(Integer)pm.evaluatePolicy(policyName,objectMap);
						if (logger.isLoggable(Level.FINE))
                        logger.fine(Thread.currentThread().getName()+" return value is = "+retVal);
                        
                        logger.exiting(sourceClass,Thread.currentThread().getName()+" "+"InvokePolicy"); 
                        
                        if (retVal.intValue() ==POLICY_EVALUATION_FAILED )
                        {
                            logger.fine("POLICY_EVALUATION_FAILED || POLICY_NOT_EVALUATED "+policyName+" "+copI.toString()+" "+retVal);
                            returnValue++;
                        }
                        else
                        {
                            logger.fine("POLICY_EVALUATION_SUCCESSFUL "+policyName+" "+copI.toString()+" "+retVal);
                            
                        }
			            if(logger.isLoggable(Level.FINE))
                        logger.fine(Thread.currentThread().getName()+" Internal client return value="+returnValue);
                        objectMap.remove(copI);
                    }
                    logger.fine("internal client finished iterating over all instances ");
                    break;
                }
            }
            return 0;
        }
        catch(SPLException e)
        {
            logger.severe(e.getMessage());
        }
        
        logger.exiting(sourceClass,Thread.currentThread().getName()+" "+"InvokeMethod");
        if(logger.isLoggable(Level.FINE))
        logger.fine(Thread.currentThread().getName()+" Internal client return value is -1");

        return -1;
    }
    
    public Object clone() throws CloneNotSupportedException
    {
        throw new CloneNotSupportedException();
    }
}
