/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.  
 */

/**
 * @author Prashant Baliga <prabalig@in.ibm.com>
 *
 */

package org.apache.imperius.spl.datastore;

import java.util.List;
import java.util.Map;

import org.apache.imperius.spl.core.Actuator;
import org.apache.imperius.spl.core.DataCollector;
import org.apache.imperius.spl.core.PolicyInfo;
import org.apache.imperius.spl.parser.exceptions.SPLException;
import org.apache.imperius.spl.parser.statements.impl.SPLPolicy;

/**
 * Provides the ability manage the storage of policies.
 * Before using the container, it must be initialized
 * through a call to {@link #init(DataCollector, Actuator)}.  When the 
 * container is no longer needed, a call to {@link #shutdown()} should be made
 * to clear up any resources.  Policies are stored in the container with
 * a unique name.  
 *  
 * @author dawood@us.ibm.com
 *
 */
public interface PolicyDataStore {

	/**
	 * Get the policy with the given name.
	 * @param policyName
	 * @return a policy string, never null.
	 * @throws SPLException 
	 */
	public String getPolicy(String policyName) throws SPLException;

	/**
	 * Parse the given policy with the given name and provide the information about the policy.
	 * @param policyName
	 * @return
	 * @throws SPLException
	 */
	public PolicyInfo getPolicyInfo(String policyName) throws SPLException;

	/**
	 * Store the given policy and give it the given name.
	 * @param policyName
	 * @param policyString
	 * @return true if stored successfully.
	 * @throws SPLException
	 */
	public boolean createPolicy(String policyName, String policyString)	throws SPLException;

    /**
     * Determine if the given policy string is properly formatted and parsable.
     * @param policyString the SPL policy.
     * @return true if the the string is parsable, false otherwise.
     * @throws SPLException // TODO: why do we need to throw an exception here?
     */
	public boolean validatePolicyString(String policyString) throws SPLException;

	/**
	 * Store the policy using the given name and create the Java object representing 
	 * a parsed policy.
	 * @param policyName
	 * @param policyString
	 * @return
	 * @throws SPLException
	 */
	public SPLPolicy createSPLPolicyObject(String policyName,
			String policyString) throws SPLException;

	/**
	 * Replace the definition of the policy with the given name.
	 * @param policyName
	 * @param policyString new policy definition.
	 * @throws SPLException
	 */
	public void updatePolicy(String policyName, String policyString) throws SPLException;

	/**
	 * Remove the named policy from the repository.
	 * @param policyName name of the policy.
	 * @throws SPLException if the named policy does not exist.
	 */
	public void deletePolicy(String policyName) throws SPLException;

	/**
	 * Get a map of all policies mapped from the name of the policy to the policy string.
	 * @return
	 * @throws SPLException
	 */
	public Map getAllPolicies() throws SPLException;

	public List getAllPolicyNames() throws SPLException;

	/**
	 * Initialize the storage.
	 * Calls to access/add policies before this may throw exceptions or otherwise fail.
	 * @throws SPLException
	 */
	public void init() throws SPLException;

	/**
	 * Signal that the storage is no longer needed.
	 * Calls to access/add policies after this call and before a call
	 * to {@link #init()} may fail.
	 */
	public void shutdown();

	/**
	 * Get the java object representation of the parsed policy associated with the given name.
	 * @param PolicyRuleName
	 * @return
	 * @throws SPLException
	 */
	public SPLPolicy getInternalPolicyObject(String PolicyRuleName)	throws SPLException;

}
