/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.  
 */

/**
 * @author Prashant Baliga <prabalig@in.ibm.com>
 *
 */

package org.apache.imperius.spl.datastore.impl;

import java.util.List;
import java.util.Map;
import java.util.logging.Logger;

import org.apache.imperius.spl.core.PolicyInfo;
import org.apache.imperius.spl.datastore.PolicyDataStore;
import org.apache.imperius.spl.parser.exceptions.InvalidPolicyParameterException;
import org.apache.imperius.spl.parser.exceptions.PolicyAlreadyExistsInRepositoryException;
import org.apache.imperius.spl.parser.exceptions.PolicyDoesNotExistException;
import org.apache.imperius.spl.parser.exceptions.SPLException;
import org.apache.imperius.spl.parser.statements.impl.PolicyInfoImpl;
import org.apache.imperius.spl.parser.statements.impl.SPLPolicy;
import org.apache.imperius.util.Messages;
import org.apache.imperius.util.SPLLogger;

/**
 * Implementation of PolicyDataStore that stores policies using serialization to 
 * the file PolicyStorage.DAT in the SPLLogger's home directory.
 * 
 * @author dawood@us.ibm.com
 *
 */
public class PolicyDataStoreImpl implements PolicyDataStore 
{
	private PolicyCache _policyCache = null;

	private PolicyParser _policyParser = null;

	private PolicyRepository _policyRepository = null;

	private static final String sourceClass = "PolicyDataStoreImpl";

	private static Logger logger = SPLLogger.getSPLLogger().getLogger();

	private static PolicyDataStoreImpl singletonObj = null;

	private PolicyDataStoreImpl() 
	{
		init();
	}

	public void init() 
	{
		logger.entering(sourceClass, Thread.currentThread().getName() + " "
				+ "init");

		_policyCache = PolicyCache.getInstance();
		_policyRepository = PolicyRepositoryImpl.getInstance();
		_policyParser = PolicyParserImpl.getInstance();

		logger.exiting(sourceClass, Thread.currentThread().getName() + " "
				+ "init");
	}

	public static synchronized PolicyDataStoreImpl getInstance() 
	{
		if (singletonObj == null) 
		{
			singletonObj = new PolicyDataStoreImpl();
		}
		return singletonObj;
	}

	public void shutdown() 
	{
		logger.entering(sourceClass, Thread.currentThread().getName() + " "
				+ "shutdown");
		
		_policyParser.shutDown();
		_policyRepository.shutdown();
		
		logger.exiting(sourceClass, Thread.currentThread().getName() + " "
				+ "shutdown");
	}

	public boolean createPolicy(String policyName, String policyString)
			throws SPLException 
	{
		logger.entering(sourceClass, Thread.currentThread().getName() + " "
				+ "createPolicy");

		boolean res = false;
		if ((policyName != null) && (policyString != null)) 
		{
			// check if Policy already exists in Repository
			if (!_policyRepository.policyExists(policyName)) 
			{
				SPLPolicy internalPolicyObject = _policyParser
						.createInternalPolicyObject(policyName, policyString);
				if (internalPolicyObject != null) 
				{
					logger
							.fine(Thread.currentThread().getName()
									+ " "
									+ Messages
											.getString("SPL_CREATED_INTERNAL_POLICY_OBJ_MSG"));

					res = _policyRepository.insertPolicy(policyName, policyString);

					logger
							.fine(Thread.currentThread().getName()
									+ " "
									+ Messages
											.getString("SPL_POLICY_INSERTION_INTO_REPOSITORY_MSG"));

					_policyCache.insertPolicy(policyName, internalPolicyObject);
				}
			} 
			else 
			{
				throw new PolicyAlreadyExistsInRepositoryException(Messages
						.getString("SPL_POLICY_ALREADY_EXISTS_MSG"));
			}
		} 
		else 
		{
			throw new InvalidPolicyParameterException(Messages
					.getString("SPL_NULL_POLICY_NAME_EXCEPTION_MSG"));
		}
		return res;
	}

	public void deletePolicy(String policyName) throws SPLException 
	{
		logger.entering(sourceClass, Thread.currentThread().getName() + " "
				+ "deletePolicy");

		if (policyName != null) 
		{
			logger.fine(Thread.currentThread().getName() + " "
					+ Messages.getString("SPL_DELETE_POLICY_FROM_CACHE_MSG"));

			// delete Policy from cache
			_policyCache.deletePolicy(policyName);

			logger
					.fine(Thread.currentThread().getName()
							+ " "
							+ Messages
									.getString("SPL_DELETE_POLICY_FROM_REPOSITORY_MSG"));

			// delete policy from Repository
			if (_policyRepository.policyExists(policyName)) 
			{
				_policyRepository.deletePolicy(policyName);
			
				logger.exiting(sourceClass, Thread.currentThread().getName()
						+ " " + "deletePolicy");
			} 
			else 
			{
				throw new PolicyDoesNotExistException(Messages
						.getString("SPL_POLICY_DOES_NOT_EXIST_EXCEPTION_MSG"));
			}
		} 
		else 
		{
			throw new InvalidPolicyParameterException(Messages
					.getString("SPL_NULL_POLICY_NAME_EXCEPTION_MSG"));
		}
	}

	public Map getAllPolicies() throws SPLException 
	{
		logger.entering(sourceClass, Thread.currentThread().getName() + " "
				+ "getAllPolicies");

		Map policies = _policyRepository.getAllPolicies();

		logger.exiting(sourceClass, Thread.currentThread().getName() + " "
				+ "getAllPolicies");

		return policies;
	}

	public List getAllPolicyNames() throws SPLException 
	{
		logger.entering(sourceClass, Thread.currentThread().getName() + " "
				+ "getAllPolicyNames");
		
		List policyNames = _policyRepository.getAllPolicyNames();

		logger.exiting(sourceClass, Thread.currentThread().getName() + " "
				+ "getAllPolicyNames");

		return policyNames;
	}

	public synchronized SPLPolicy getInternalPolicyObject(String policyRuleName)
			throws SPLException 
	{
		logger.entering(sourceClass, Thread.currentThread().getName() + " "
				+ "getInternalPolicyObject");
		
		if (policyRuleName != null) 
		{
			if (_policyCache.policyExists(policyRuleName)) 
			{
				// get policy object from cache if exists
				SPLPolicy policyObject = _policyCache.getPolicy(policyRuleName);
			
				logger.exiting(sourceClass, Thread.currentThread().getName()
						+ " " + "getInternalPolicyObject");
				
				return policyObject;
			}
			else if (_policyRepository.policyExists(policyRuleName)) 
			{
				// get policy string from repository
				String PolicyString = _policyRepository
						.getPolicyString(policyRuleName);
				
				// create internal policy object from the policy string
				SPLPolicy internalPolicyObject = _policyParser
						.createInternalPolicyObject(policyRuleName,
								PolicyString);
				
				// insert the policy into internal cache
				_policyCache.insertPolicy(policyRuleName, internalPolicyObject);

				logger.exiting(sourceClass, Thread.currentThread().getName()
						+ " " + "getInternalPolicyObject");
				
				return internalPolicyObject;
			} 
			else 
			{
				throw new PolicyDoesNotExistException(Messages
						.getString("SPL_POLICY_DOES_NOT_EXIST_EXCEPTION_MSG"));
			}
		} 
		else 
		{
			throw new InvalidPolicyParameterException(Messages
					.getString("SPL_NULL_POLICY_NAME_EXCEPTION_MSG"));
		}
	}

	public String getPolicy(String policyRuleName) throws SPLException 
	{
		logger.entering(sourceClass, Thread.currentThread().getName() + " "
				+ "getPolicy");

		if (policyRuleName != null) 
		{
			// check to see if Policy exists, if so then return it
			if (_policyRepository.policyExists(policyRuleName)) 
			{
				String policyString = _policyRepository.getPolicyString(policyRuleName);
				
				logger.exiting(sourceClass, Thread.currentThread().getName()
						+ " " + "getPolicy");
				
				return policyString;
			} 
			else 
			{
				throw new PolicyDoesNotExistException(Messages
						.getString("SPL_POLICY_DOES_NOT_EXIST_EXCEPTION_MSG"));
			}
		} 
		else 
		{
			throw new InvalidPolicyParameterException(Messages
					.getString("SPL_NULL_POLICY_NAME_EXCEPTION_MSG"));
		}
	}

	public void updatePolicy(String policyRuleName, String policyString)
			throws SPLException 
	{
		logger.entering(sourceClass, Thread.currentThread().getName() + " "
				+ "updatePolicy");

		if ((policyRuleName != null) && (policyString != null)) 
		{
			if (_policyRepository.policyExists(policyRuleName)) 
			{
				// create internal policy from the policy string
				SPLPolicy internalPolicyObject = _policyParser.createInternalPolicyObject(policyRuleName, policyString);
				if (internalPolicyObject != null) 
				{
					// update the policy repository
					_policyRepository.updatePolicy(policyRuleName, policyString);
					
					// update the internal cache with the current policy object
					if (_policyCache.policyExists(policyRuleName)) 
					{
						logger.fine(Thread.currentThread().getName()
								+ " deleting old Policy Object from cache");
						
						_policyCache.deletePolicy(policyRuleName);
					}
					
					logger.fine(Thread.currentThread().getName()
							+ " adding new Policy Object to cache");

					_policyCache.insertPolicy(policyRuleName, internalPolicyObject);

					logger.exiting(sourceClass, Thread.currentThread()
							.getName()
							+ " " + "updatePolicy");
				}
			} 
			else 
			{
				throw new PolicyDoesNotExistException(Messages
						.getString("SPL_POLICY_DOES_NOT_EXIST_EXCEPTION_MSG"));
			}
		} 
		else 
		{
			throw new InvalidPolicyParameterException(Messages
					.getString("SPL_NULL_POLICY_NAME_EXCEPTION_MSG"));
		}
	}

	public PolicyInfo getPolicyInfo(String policyRuleName) throws SPLException 
	{
		logger.entering(sourceClass, Thread.currentThread().getName() + " "
				+ "getPolicyInfo");

		if (policyRuleName != null) 
		{
			// check if policy exists in cache
			if (_policyCache.policyExists(policyRuleName)) 
			{
				SPLPolicy policyObject = _policyCache.getPolicy(policyRuleName);
				List importStmtList = policyObject.getImportStmtList();

				logger.exiting(sourceClass, Thread.currentThread().getName()
						+ " " + "getPolicyInfo");
				
				PolicyInfo pi = new PolicyInfoImpl(importStmtList);
				return pi;
			}
			else if (_policyRepository.policyExists(policyRuleName)) 
			{
				String PolicyString = _policyRepository.getPolicyString(policyRuleName);

				SPLPolicy internalPolicyObject = _policyParser.createInternalPolicyObject(policyRuleName, PolicyString);
				_policyCache.insertPolicy(policyRuleName, internalPolicyObject);
				
				List importStmtList = internalPolicyObject.getImportStmtList();
				PolicyInfo pi = new PolicyInfoImpl(importStmtList);

				logger.exiting(sourceClass, Thread.currentThread().getName()
						+ " " + "getPolicyInfo");

				return pi;
			} 
			else 
			{
				throw new PolicyDoesNotExistException(Messages
						.getString("SPL_POLICY_DOES_NOT_EXIST_EXCEPTION_MSG"));
			}
		} 
		else 
		{
			throw new InvalidPolicyParameterException(Messages
					.getString("SPL_NULL_POLICY_NAME_EXCEPTION_MSG"));
		}
	}

	public SPLPolicy createSPLPolicyObject(String policyName,
			String policyString) throws SPLException 
	{
		logger.entering(sourceClass, Thread.currentThread().getName() + " "
				+ "createSPLPolicyObject");

		if ((policyName != null) && (policyString != null)) 
		{
			// check if Policy already exists in Repository
			SPLPolicy internalPolicyObject = _policyParser
					.createInternalPolicyObject(policyName, policyString);
			
			if (internalPolicyObject != null) 
			{
				logger.fine(Thread.currentThread().getName()
						+ " internal Policy object created successfully");

				return internalPolicyObject;
			}
		} 
		else 
		{
			throw new InvalidPolicyParameterException(Messages
					.getString("SPL_NULL_POLICY_NAME_EXCEPTION_MSG"));
		}
		return null;
	}

	public boolean validatePolicyString(String policyString)
			throws SPLException 
	{
		logger.entering(sourceClass, Thread.currentThread().getName() + " "
				+ "validatePolicyString");
		
		boolean pi = this._policyParser.validatePolicyString(policyString);
		
		logger.exiting(sourceClass, Thread.currentThread().getName() + " "
				+ "validatePolicyString");

		return pi;
	}

}
