/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.  
 */

/**
 * @author Prashant Baliga <prabalig@in.ibm.com>
 *
 */

package org.apache.imperius.spl.datastore.impl;

import java.io.Reader;
import java.io.StringReader;
import java.util.logging.Level;
import java.util.logging.Logger;

import org.apache.imperius.spl.core.DataCollector;
import org.apache.imperius.spl.parser.compiler.SPLLexer;
import org.apache.imperius.spl.parser.compiler.SPLParser;
import org.apache.imperius.spl.parser.compiler.SPLTreeParser;
import org.apache.imperius.spl.parser.compiler.symboltable.SPLSymbolTable;
import org.apache.imperius.spl.parser.exceptions.SPLException;
import org.apache.imperius.spl.parser.statements.impl.SPLPolicy;
import org.apache.imperius.spl.parser.util.DataCollectorFactory;
import org.apache.imperius.util.Messages;
import org.apache.imperius.util.SPLLogger;

import antlr.RecognitionException;
import antlr.collections.AST;

public class PolicyParserImpl implements PolicyParser 
{
	private static final String sourceClass = "PolicyParserImpl";

	private static Logger logger = SPLLogger.getSPLLogger().getLogger();

	static boolean showTree = false;

	private static PolicyParserImpl singletonObj = null;

	private PolicyParserImpl() 
	{
		init();
	}

	public void init() 
	{
		logger.entering(sourceClass, Thread.currentThread().getName() + " "
				+ "init");

		logger.exiting(sourceClass, Thread.currentThread().getName() + " "
				+ "init");
	}

	public static synchronized PolicyParserImpl getInstance() 
	{
		if (singletonObj == null) 
		{
			singletonObj = new PolicyParserImpl();
		}
		return singletonObj;
	}

	public void shutDown() 
	{
		logger.entering(sourceClass, Thread.currentThread().getName() + " "
				+ "shutDown");
		
		logger.exiting(sourceClass, Thread.currentThread().getName() + " "
				+ "shutDown");
	}

	public synchronized SPLPolicy createInternalPolicyObject(
			String policyRuleName, String policyString) throws SPLException 
	{
		logger.entering(sourceClass, Thread.currentThread().getName() + " "
				+ "CreateInternalPolicyObject");
		StringReader policyStringReader = new StringReader(policyString);
		SPLPolicy pol = parseFile(policyRuleName, policyStringReader);
		
		logger.exiting(sourceClass, Thread.currentThread().getName() + " "
				+ "CreateInternalPolicyObject");
		
		return pol;
	}

	public boolean validate(String PolicyRuleName, String PolicyString) 
	{
		logger.entering(sourceClass, Thread.currentThread().getName() + " "
				+ "validate");
		
		logger.exiting(sourceClass, Thread.currentThread().getName() + " "
				+ "validate");

		return true;
	}

	public static SPLPolicy parseFile(String f, Reader r) throws SPLException 
	{
		logger.entering(sourceClass, Thread.currentThread().getName() + " "
				+ "parseFile");

		try 
		{
			// Create a scanner that reads from the input stream passed to us
			SPLLexer lexer = new SPLLexer(r);
			lexer.setFilename(f);

			// Create a parser that reads from the scanner
			SPLParser parser = new SPLParser(lexer);
			parser.setASTNodeClass("org.apache.imperius.spl.parser.compiler.ASTWithLineNumber");
			parser.setFilename(f);
			
			logger.fine(Thread.currentThread().getName() + " parsing file started...");

			// start parsing at the compilationUnit rule
			parser.splpolicy();
			
			if(parser.getErrorList() != null && parser.getErrorList().size() > 0)
			{
				throw new SPLException(Messages
						.getString("SPL_POLICY_PARSING_EXCEPTION_MSG"));
			}
			
			logger.fine(Thread.currentThread().getName() + " parsing file completed here...");
			
			AST t = parser.getAST();
			
			if (t != null) 
			{
				traverseTree(t);
			} 
			else 
			{
				logger.log(Level.SEVERE, Messages.SPLOF0007E);
			}

			// do something with the tree
			SPLPolicy pol = doTreeAction(f, parser.getAST(), parser.getTokenNames());
			
			logger.exiting(sourceClass, Thread.currentThread().getName() + " "
					+ "parseFile");
			
			return pol;
		} 
		catch (Exception e) 
		{
			logger.severe(e.getMessage());
			throw new SPLException(e.getMessage());
		}
	}

	public static void traverseTree(AST t) 
	{

	}

	public static synchronized SPLPolicy doTreeAction(String f, AST t,
			String[] tokenNames) throws SPLException 
	{
		logger.entering(sourceClass, Thread.currentThread().getName() + " "
				+ "doTreeAction");

		if (t == null) 
		{
			return null;
		}

		SPLTreeParser tparse = new SPLTreeParser();
		tparse.setASTNodeClass("org.apache.imperius.spl.parser.compiler.ASTWithLineNumber");
		try 
		{
			DataCollector dc = DataCollectorFactory.getDataCollector();
			SPLSymbolTable st = new SPLSymbolTable();
			SPLPolicy cp = tparse.splpolicy(t, dc, st);
			if(tparse.getErrorList() != null && tparse.getErrorList().size() > 0)
			{
				throw new SPLException(Messages.getString("SPL_POLICY_TREE_PARSING_EXCEPTION_MSG"));
			}
			logger.exiting(sourceClass, Thread.currentThread().getName() + " "
					+ "doTreeAction");
			return cp;
		}
		catch (RecognitionException e) 
		{
			throw new SPLException(e.getMessage());
		}
	}

	public boolean validatePolicyString(String policyString)
			throws SPLException 
	{
		logger.entering(sourceClass, Thread.currentThread().getName() + " "
				+ "validatePolicyString");
		
		try 
		{
			StringReader policyStringReader = new StringReader(policyString);
			parseFile("validate", policyStringReader);
		
			logger.exiting(sourceClass, Thread.currentThread().getName() + " "
					+ "CreateInternalPolicyObject");

			return true;
		} 
		catch (Exception e) 
		{
			logger.log(Level.SEVERE, Messages.SPLOF0006E, new Object[] { e
					.getMessage() });
			return false;
		}

	}
}
