/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.  
 */

/**
 * @author Prashant Baliga <prabalig@in.ibm.com>
 *
 * Modified : 11/1/07 
 * 
 */

package org.apache.imperius.spl.datastore.impl;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.logging.Level;
import java.util.logging.Logger;

import org.apache.imperius.spl.parser.exceptions.InvalidPolicyParameterException;
import org.apache.imperius.spl.parser.exceptions.PolicyAlreadyExistsInRepositoryException;
import org.apache.imperius.spl.parser.exceptions.PolicyDoesNotExistException;
import org.apache.imperius.spl.parser.exceptions.SPLException;
import org.apache.imperius.util.Messages;
import org.apache.imperius.util.SPLLogger;

class PolicyRepositoryImpl implements PolicyRepository 
{
	private static HashMap _policyMap = new HashMap();
	private static String _fileName = null;
	private static Logger logger = SPLLogger.getSPLLogger().getLogger();
	private static final String sourceClass = "PolicyRepositoryImpl";
	private static PolicyRepositoryImpl singletonObj = null;

	private PolicyRepositoryImpl() 
	{
		init();
	}

	public void init() 
	{
		logger.entering(sourceClass, Thread.currentThread().getName() + " "
				+ "init");

		_fileName = SPLLogger.SPL_HOME + SPLLogger.fileSeparator + REPOSITORY_FILE_NAME;
		_readSerializedPolicyStorage();
		
		logger.exiting(sourceClass, Thread.currentThread().getName() + " "
				+ "init");
	}

	public static synchronized PolicyRepositoryImpl getInstance() 
	{
		if (singletonObj == null) 
		{
			singletonObj = new PolicyRepositoryImpl();
		}
		return singletonObj;
	}

	public void shutdown() 
	{
		logger.entering(sourceClass, Thread.currentThread().getName() + " "
				+ "shutdown");

		logger.exiting(sourceClass, Thread.currentThread().getName() + " "
				+ "shutdown");
	}

	public Map getAllPolicies() throws SPLException 
	{
		logger.entering(sourceClass, Thread.currentThread().getName() + " "
				+ "getAllPolicies");
		
		Map allPolicies = new Hashtable();
		_readSerializedPolicyStorage();
		Iterator instanceKeySet = _policyMap.keySet().iterator();
		while (instanceKeySet.hasNext()) 
		{
			String key = (String) instanceKeySet.next();
			String policyString = (String) _policyMap.get(key);
			allPolicies.put(key, policyString);
		}

		logger.exiting(sourceClass, Thread.currentThread().getName() + " "
				+ "getAllPolicies");
		return allPolicies;
	}

	public List getAllPolicyNames() throws SPLException 
	{
		logger.entering(sourceClass, Thread.currentThread().getName() + " "
				+ "getAllPolicyNames");
		
		List policyNames = new ArrayList();
		_readSerializedPolicyStorage();
		Iterator instanceKeySet = _policyMap.keySet().iterator();
		while (instanceKeySet.hasNext()) 
		{
			// add all Policy names from policyMap to PolicyNames and return it
			String policyName = (String) instanceKeySet.next();
			policyNames.add(policyName);
		}

		logger.exiting(sourceClass, Thread.currentThread().getName() + " "
				+ "getAllPolicyNames");
		
		return policyNames;
	}

	public void updatePolicy(String policyRuleName, String policyString)
			throws SPLException 
	{
		logger.entering(sourceClass, Thread.currentThread().getName() + " "
				+ "updatePolicy");
		
		_readSerializedPolicyStorage();
		if (policyRuleName != null && policyString != null) 
		{
			if (_policyMap.containsKey(policyRuleName)) 
			{
				logger.fine(Thread.currentThread().getName()
						+ " updating policy in repository");

				// updating Repository
				deletePolicy(policyRuleName);
				insertPolicy(policyRuleName, policyString);
				_serializePolicyStorage();
				
				logger.exiting(sourceClass, Thread.currentThread().getName()
						+ " " + "updatePolicy");
			} 
			else 
			{
				throw new PolicyDoesNotExistException(Messages
						.getString("SPL_POLICY_DOES_NOT_EXIST_EXCEPTION_MSG"));
			}
		} 
		else 
		{
			throw new InvalidPolicyParameterException(Messages
					.getString("SPL_NULL_POLICY_NAME_EXCEPTION_MSG"));
		}
	}

	public String getPolicyString(String policyRuleName) throws SPLException 
	{
		logger.entering(sourceClass, Thread.currentThread().getName() + " "
				+ "getPolicyString");
		
		_readSerializedPolicyStorage();
		if (policyRuleName != null) 
		{
			if (_policyMap.containsKey(policyRuleName)) 
			{
				logger.fine(Thread.currentThread().getName()
						+ " return Policy from policyMap");

				// return Policy from policyMap
				String policy = (String) _policyMap.get(policyRuleName);

				logger.exiting(sourceClass, Thread.currentThread().getName()
						+ " " + "getPolicy");
				
				return policy;
			} 
			else 
			{
				throw new PolicyDoesNotExistException(Messages
						.getString("SPL_POLICY_DOES_NOT_EXIST_EXCEPTION_MSG"));
			}
		} 
		else 
		{
			throw new InvalidPolicyParameterException(Messages
					.getString("SPL_NULL_POLICY_NAME_EXCEPTION_MSG"));
		}
	}

	public synchronized boolean insertPolicy(String policyRuleName,
			String policyString) throws SPLException 
	{
		logger.entering(sourceClass, Thread.currentThread().getName() + " "
				+ "insertPolicy");

		_readSerializedPolicyStorage();
		if (policyRuleName != null && policyString != null) 
		{
			if (_policyMap.containsKey(policyRuleName)) 
			{
				throw new PolicyAlreadyExistsInRepositoryException(Messages
						.getString("SPL_POLICY_ALREADY_EXISTS_MSG"));
			} 
			else 
			{
				logger.fine(Thread.currentThread().getName()
						+ " add policy to policyMap ");

				// add policy to policyMap
				_policyMap.put((String) policyRuleName, (String) policyString);
				_serializePolicyStorage();
				return true;
			}
		} 
		else 
		{
			throw new InvalidPolicyParameterException(Messages
					.getString("SPL_NULL_POLICY_NAME_EXCEPTION_MSG"));
		}
	}

	/**
	 * removes a policy from the Hashtable
	 * 
	 * @param PolicyRuleName
	 * @throws SPLException
	 */
	public void deletePolicy(String policyRuleName) throws SPLException 
	{
		logger.entering(sourceClass, Thread.currentThread().getName() + " "
				+ "deletePolicy");

		_readSerializedPolicyStorage();
		if (policyRuleName != null) 
		{
			if (!_policyMap.containsKey(policyRuleName)) 
			{
				throw new PolicyDoesNotExistException(Messages
						.getString("SPL_POLICY_DOES_NOT_EXIST_EXCEPTION_MSG"));
			} 
			else 
			{
				logger.fine(Thread.currentThread().getName()
						+ " removing policy from policyMap");

				// removing policy from policyMap
				_policyMap.remove(policyRuleName);
				_serializePolicyStorage();
				
				logger.exiting(sourceClass, Thread.currentThread().getName()
						+ " " + "deletePolicy");
			}
		} 
		else 
		{
			throw new InvalidPolicyParameterException(Messages
					.getString("SPL_NULL_POLICY_NAME_EXCEPTION_MSG"));
		}
	}

	public boolean policyExists(String PolicyRuleName) throws SPLException 
	{
		logger.entering(sourceClass, Thread.currentThread().getName() + " "
				+ "policyExists");
		
		_readSerializedPolicyStorage();
		if (PolicyRuleName != null) 
		{
			logger.fine(Thread.currentThread().getName()
					+ " check to see if Policy exists in policyMap");

			// check to see if Policy exists in policyMap
			if (_policyMap.containsKey(PolicyRuleName)) {

				logger.exiting(sourceClass, Thread.currentThread().getName()
						+ " " + "policyExists");
				
				return true;
			}
			else 
			{
				logger.exiting(sourceClass, Thread.currentThread().getName()
						+ " " + "policyExists");
				return false;
			}
		} 
		else 
		{
			throw new InvalidPolicyParameterException(Messages
					.getString("SPL_NULL_POLICY_NAME_EXCEPTION_MSG"));
		}
	}

	private synchronized static void _readSerializedPolicyStorage() 
	{
		logger.entering(sourceClass, Thread.currentThread().getName() + " "
				+ "_readSerializedPolicyStorage");
		
		FileInputStream fis = null;
		ObjectInputStream in = null;
		File file = new File(_fileName);
		try 
		{
			logger
					.fine(Thread.currentThread().getName()
							+ " read from the serialized file if it exists, else create it and serialize an empty policyMap to it");

			// read from the serialized file if it exists, else create it and
			// serialize an empty policyMap to it
			if (!file.exists()) {

				FileOutputStream fos = new FileOutputStream(file);
				ObjectOutputStream out = new ObjectOutputStream(fos);
				out.writeObject(_policyMap);
				out.close();
				fos.close();
			}
			fis = new FileInputStream(file);
			in = new ObjectInputStream(fis);
			_policyMap = (HashMap) in.readObject();

			in.close();
			fis.close();

			logger.exiting(sourceClass, Thread.currentThread().getName() + " "
					+ "_readSerializedPolicyStorage");
		} 
		catch (IOException ex) 
		{
			logger.log(Level.SEVERE, Messages.SPLOF0003E);
		} 
		catch (ClassNotFoundException ex) 
		{
			logger.log(Level.SEVERE, Messages.SPLOF0004E);
		}
	}

	private synchronized static void _serializePolicyStorage() 
	{
		logger.entering(sourceClass, Thread.currentThread().getName() + " "
				+ "_serializePolicyStorage");
		
		FileOutputStream fos = null;
		ObjectOutputStream out = null;
		File file = new File(_fileName);
		try 
		{
			logger.fine(Thread.currentThread().getName()
					+ " serialize an policyMap to File System");

			// serialize an policyMap to File System
			fos = new FileOutputStream(file);
			out = new ObjectOutputStream(fos);
			out.writeObject(_policyMap);
			out.close();
			fos.close();

			logger.exiting(sourceClass, Thread.currentThread().getName() + " "
					+ "_serializePolicyStorage");
		} 
		catch (IOException ex) 
		{
			logger.log(Level.SEVERE, Messages.SPLOF0005E);
		}
	}

}
