/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.  
 */


/**
 * @author Prashant Baliga <prabalig@in.ibm.com>
 * 
 * This is the primary interface into the SPL Engine
 *
 */

package org.apache.imperius.spl.manager;

import java.io.IOException;
import java.util.List;
import java.util.Map;

import org.apache.imperius.spl.core.Actuator;
import org.apache.imperius.spl.core.DataCollector;
import org.apache.imperius.spl.core.PolicyInfo;
import org.apache.imperius.spl.parser.exceptions.SPLException;

/**
 * Provides the ability manage the storage of policies and to evaluate policies
 * contained in the container.  Before using the container, it must be initialized
 * through a call to {@link #init(DataCollector, Actuator)}.  When the 
 * container is no longer needed, a call to {@link #shutdown()} should be made
 * to clear up any resources.  Policies are stored in the container with
 * a unique name.  Once stored, policies may be evaluated by name.
 *  
 * @author dawood@us.ibm.com
 *
 */
public interface PolicyManager
{
    /**
     * Returns the policy as a String
     * @param name
     * @return
     * @throws SPLException
     */
    public String getPolicy(String name)
    	throws SPLException;
    
    /**
     * Returns a policyInfo object that contains the anchor class names and 
     * qualifiers.
     * @param policyName
     * @return
     * @throws SPLException
     */
    public PolicyInfo getPolicyInfo(String policyName)
    	throws SPLException;
    
    /**
     * Creates a policy with the specified name and string
     * @param policyName
     * @param policyString
     * @return
     * @throws SPLException
     */
    public boolean createPolicy(String policyName, String policyString) 
    	throws SPLException;
    
    /**
     * Updates the policy with the given name with the provided string.
     * @param policyName
     * @param policyString
     * @throws SPLException
     */
    public void updatePolicy(String policyName, String policyString) 
    	throws SPLException;
    
    /**
     * Deletes the policy with the given name
     * @param policyName
     * @throws SPLException
     */
    public void deletePolicy(String policyName)
    	throws SPLException;
    
    /**
     * Returns a map (policyName -> policy String) of all policies 
     * stored in the repository
     * @return
     * @throws SPLException
     */
    public Map getAllPolicies()
    	throws SPLException;
    
    /**
     * Gets a list of names of all policies in the system
     * @return
     * @throws SPLException
     */
    public List getAllPolicyNames() throws SPLException;
    
    /**
     * Evaluates the policy with the given name. The input map should be of the
     * following format
     * <fully qualified classname> -> List of InstanceInfo objects
     * 
     * Where InstanceInfo object contains <instance name> and an empty object
     * @param policyName
     * @param instance
     * @return
     * @throws SPLException
     */
    
    public Object evaluatePolicy(String policyName, Map instance) 
    	throws SPLException;
    
    /**
     * Initializes the instance wit the given DataCollector and Actuator
     * @param dc
     * @param ac
     * @throws SPLException
     * @throws IOException
     */
    public void init(DataCollector dc, Actuator ac) 	throws SPLException, IOException;
    
//    public void setSPLHome(String location);
//    
//    public String getSPLHome();
    
    /**
     * Place the given policy into the container and then evaluate it against the 
     * given instance data.
     * @param policyName name to be assigned in the container
     * @param policyString SPL policy 
     * @param instances the instance data to be applied to the policy.
     * @return TODO
     * @throws SPLException
     */
    public Object evaluatePolicyString(String policyName, String policyString, Map instances) throws SPLException; 
	
    /**
     * Set the whether logging should be on or off.
     * 
     * @param enableOrDisable
     */
    public void enableLogging(boolean enableOrDisable);
    
    /**
     * Shutdown  this instance.
     * Evaluation requests made after this call and before another call to {@link #init(DataCollector, Actuator)}
     * may cause exceptions.
     *
     */
    public void shutdown();
    
    /**
     * Determine if the given policy string is properly formatted and parsable.
     * @param policyString the SPL policy.
     * @return true if the the string is parsable, false otherwise.
     * @throws SPLException // TODO: why do we need to throw an exception here?
     */
    public boolean validatePolicyString(String policyString) throws SPLException;
    
 
}
