/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.  
 */

/**
 * @author Prashant Baliga <prabalig@in.ibm.com>
 *
 */

package org.apache.imperius.spl.manager.impl;

import java.util.List;
import java.util.Map;
import java.util.logging.Level;
import java.util.logging.Logger;

import org.apache.imperius.spl.core.Actuator;
import org.apache.imperius.spl.core.DataCollector;
import org.apache.imperius.spl.core.PolicyInfo;
import org.apache.imperius.spl.datastore.PolicyDataStore;
import org.apache.imperius.spl.datastore.impl.PolicyDataStoreImpl;
import org.apache.imperius.spl.evaluator.internal.PolicyEvaluator;
import org.apache.imperius.spl.evaluator.internal.impl.PolicyEvaluatorImpl;
import org.apache.imperius.spl.manager.PolicyManager;
import org.apache.imperius.spl.parser.exceptions.SPLException;
import org.apache.imperius.spl.parser.statements.impl.SPLPolicy;
import org.apache.imperius.spl.parser.util.ActuatorFactory;
import org.apache.imperius.spl.parser.util.DataCollectorFactory;
import org.apache.imperius.util.Messages;
import org.apache.imperius.util.SPLLogger;

public class PolicyManagerImpl implements PolicyManager 
{
	private PolicyDataStore _policyDataStore = null;
	private PolicyEvaluator _policyEvaluator = null;
	private static Logger logger = SPLLogger.getSPLLogger().getLogger();
	private static final String sourceClass = "PolicyManagerImpl";
	private static Logger auditLogger = SPLLogger.getSPLLogger().getAuditLogger();

	public PolicyManagerImpl(DataCollector dc, Actuator ac) 
	{
		init(dc, ac);
	}

	public void init(DataCollector dc, Actuator ac) 
	{
		ActuatorFactory.registerActuator(ac);
		DataCollectorFactory.registerDataCollector(dc);

		_policyDataStore = PolicyDataStoreImpl.getInstance();
		_policyEvaluator = PolicyEvaluatorImpl.getInstance();
	}

	public Map getAllPolicies() throws SPLException 
	{
		logger.entering(sourceClass, Thread.currentThread().getName() + " " + "getAllPolicies");
		
		Map allPolicies = _policyDataStore.getAllPolicies();
		if(auditLogger.isLoggable(Level.FINE))
		auditLogger.fine(Thread.currentThread().getName() + "getAllPolicies invoked");

		logger.exiting(sourceClass, Thread.currentThread().getName() + " " + "getAllPolicies");
		
		return allPolicies;
	}

	public List getAllPolicyNames() throws SPLException 
	{
		logger.entering(sourceClass, Thread.currentThread().getName() + " "	+ "getAllPolicyNames");

		List allPolicyNames = _policyDataStore.getAllPolicyNames();
		if(auditLogger.isLoggable(Level.FINE))
			auditLogger.fine(Thread.currentThread().getName()+"getAllPolicyNames invoked");
		
		logger.exiting(sourceClass, Thread.currentThread().getName() + " " + "getAllPolicyNames");
		
		return allPolicyNames;
	}

	public String getPolicy(String policyName) throws SPLException 
	{
		logger.entering(sourceClass, Thread.currentThread().getName() + " "	+ "getPolicy");
		String ci = _policyDataStore.getPolicy(policyName);
		if(auditLogger.isLoggable(Level.FINE))
			auditLogger.fine(Thread.currentThread().getName()+"getPolicy invoked :"+policyName);

		logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "getPolicy");
		return ci;


	}

	public void shutdown() 
	{
		logger.entering(sourceClass, Thread.currentThread().getName() + " " + "shutdown");
		if(auditLogger.isLoggable(Level.FINE))
			auditLogger.fine(Thread.currentThread().getName() + " Policy Manager Shut Down initiated");

		logger.exiting(sourceClass, Thread.currentThread().getName() + " " + "shutdown");
	}

	public void updatePolicy(String policyName, String policyString)
			throws SPLException 
	{
		logger.entering(sourceClass, Thread.currentThread().getName() + " " + "updatePolicy");

		_policyDataStore.updatePolicy(policyName, policyString);
		if(auditLogger.isLoggable(Level.FINE))
			auditLogger.fine(Thread.currentThread().getName() + " updatePolicy method is invoked :" + policyName);

		logger.exiting(sourceClass, Thread.currentThread().getName() + " " + "updatePolicy");
	}

	public boolean createPolicy(String policyName, String policyString)
			throws SPLException 
	{
		try 
		{
			logger.entering(sourceClass, Thread.currentThread().getName() + " " + "createPolicy");

			boolean res = _policyDataStore.createPolicy(policyName, policyString);
			if(auditLogger.isLoggable(Level.FINE))
				auditLogger.fine(Thread.currentThread().getName() + " createPolicy method invoked for policy : " + policyName);

			logger.exiting(sourceClass, Thread.currentThread().getName() + " " + "createPolicy");

			return res;
		} 
		catch (SPLException e) 
		{
			logger.log(Level.SEVERE, Messages.SPLOF0001E, new Object[]{policyName});
			throw new SPLException(e.getMessage());
		}
	}

	public void deletePolicy(String policyName) throws SPLException 
	{
		try 
		{
			logger.entering(sourceClass, Thread.currentThread().getName() + " "	+ "deletePolicy");

			_policyDataStore.deletePolicy(policyName);
			if(auditLogger.isLoggable(Level.FINE))
				auditLogger.fine(Thread.currentThread().getName() + " deletePolicy method is invoked : " + policyName);

			logger.exiting(sourceClass, Thread.currentThread().getName() + " " + "deletePolicy");
		} 
		catch (SPLException e) 
		{
			logger.log(Level.SEVERE, Messages.SPLOF0002E, e.getLocalizedMessage());
			throw new SPLException(e.getLocalizedMessage());
		}
	}

	public Object evaluatePolicy(String policyName, Map instances)
			throws SPLException 
	{
		logger.entering(sourceClass, Thread.currentThread().getName() + " " + "evaluatePolicy");

		SPLPolicy internalPolicyObject = _policyDataStore.getInternalPolicyObject(policyName);

		Object result = _policyEvaluator.evaluatePolicy(internalPolicyObject, instances);

		logger.exiting(sourceClass, Thread.currentThread().getName() + " " + "evaluatePolicy");

		return result;
	}

	public PolicyInfo getPolicyInfo(String policyName) throws SPLException 
	{
		logger.entering(sourceClass, Thread.currentThread().getName() + " "	+ "getPolicyInfo");
		
		PolicyInfo pi = this._policyDataStore.getPolicyInfo(policyName);
		
		logger.exiting(sourceClass, Thread.currentThread().getName() + " " + "getPolicyInfo");

		return pi;
	}

	public Object evaluatePolicyString(String policyName, String policyString,
			Map instances) throws SPLException 
	{
		try 
		{
			logger.entering(sourceClass, Thread.currentThread().getName() + " " + "createPolicy");

			Object result;
			SPLPolicy cp = _policyDataStore.createSPLPolicyObject(policyName, policyString);
			result = _policyEvaluator.evaluatePolicy(cp, instances);
			if(auditLogger.isLoggable(Level.FINE))
			auditLogger.fine(Thread.currentThread().getName() + " evaluatePolicyString method is invoked :" + policyName);
			
			logger.exiting(sourceClass, Thread.currentThread().getName() + " " + "createPolicy");
			
			return result;
		} 
		catch (SPLException e) 
		{
			logger.log(Level.SEVERE, "Error while evaluating the policy : "+policyName);
			throw new SPLException(e.getLocalizedMessage());
		}
	}

//	public void setSPLHome(String location) 
//	{
//		SPLLogger.SPL_HOME = location;
//	}
//
//	public String getSPLHome() 
//	{
//		return SPLLogger.SPL_HOME;
//	}

	public void enableLogging(boolean enableOrDisable) 
	{
		if (enableOrDisable) 
		{
			SPLLogger.getSPLLogger().enableSPLLogger();
		} 
		else 
		{
			SPLLogger.getSPLLogger().disableSPLLogger();
		}
	}

	public boolean validatePolicyString(String policyString)
			throws SPLException 
	{
		logger.entering(sourceClass, Thread.currentThread().getName() + " "	+ "validatePolicyString");
		
		boolean pi = this._policyDataStore.validatePolicyString(policyString);

		logger.exiting(sourceClass, Thread.currentThread().getName() + " " + "validatePolicyString");

		return pi;
	}

}
