/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.  
 */
//

/**
 * @author Prashant Baliga <prabalig@in.ibm.com>
 *
 */



package org.apache.imperius.spl.parser.expression.primary;

import java.security.InvalidParameterException;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.logging.Logger;

import org.apache.imperius.spl.core.Expression;
import org.apache.imperius.spl.core.TypeInfo;
import org.apache.imperius.spl.parser.compiler.symboltable.SPLSymbolTable;
import org.apache.imperius.spl.parser.exceptions.IllegalParameterTypeException;
import org.apache.imperius.spl.parser.exceptions.SPLException;
import org.apache.imperius.spl.parser.util.TypeInfoImpl;
import org.apache.imperius.spl.parser.util.TypeResolver;
import org.apache.imperius.util.Messages;
import org.apache.imperius.util.SPLLogger;


public class BasicCollectionExpression  implements Expression 
{
	private List _elements = null;
	private static Logger logger = SPLLogger.getSPLLogger().getLogger();
	private static final String sourceClass = "BasicCollectionExpression";

	private int _size = 0;
	private TypeInfo _collectionType = null;
	private String _referenceTypeName = "";
	
	public BasicCollectionExpression(List pList, SPLSymbolTable sTab) 
	{
		logger.entering(sourceClass,Thread.currentThread().getName()+" "+ "BasicCollectionExpression");
		
		if(pList.isEmpty()) 
		{
            logger.severe(Thread.currentThread().getName()+" "+
            		"No of parameters for Basic collection has to be greater than zero ");
			
			throw new InvalidParameterException(Messages
					.getString("SPL_EXPRESSION_VALIDATION_FAIL_MSG")
					+ sourceClass); 
		}
		_elements=pList;
		try
		{
			if (sTab != null) { // xiping 03/05/09 to support collection analysis. sTab must not be null in normal cases.
				if(!validate())
				{
               	 	logger.severe(Thread.currentThread().getName()+" "+
                		"No of parameters for Basic collection has to be greater than zero ");
                	throw new IllegalParameterTypeException(Messages
						.getString("SPL_EXPRESSION_VALIDATION_FAIL_MSG")
						+ sourceClass);
				}
			}
		}
		catch(SPLException e)
		{
			e.printStackTrace();
		}
		_size = _elements.size();
		logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "BasicCollectionExpression");
	}

	public Object evaluate() throws SPLException 
	{
		logger.entering(sourceClass,Thread.currentThread().getName()+" "+ "evaluate");
		
		ArrayList resultCollection=new ArrayList();
		Iterator ExpIt = _elements.iterator();
		while(ExpIt.hasNext())
		{
			Expression exp = (Expression)ExpIt.next();
			Object resultElement = exp.evaluate();
			resultCollection.add(resultElement);
		}
		logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "evaluate");
		return resultCollection;
	}

	public boolean validate() throws SPLException 
	{
		logger.entering(sourceClass,Thread.currentThread().getName()+" "+ "validate");
		
		Iterator paramListIterator = _elements.iterator();
		while(paramListIterator.hasNext())
		{
			Expression expression=(Expression)paramListIterator.next();
			if(_collectionType == null)
			{
				_collectionType = new TypeInfoImpl(expression.getType());
				_collectionType.setIsArray(true);
				if(TypeResolver.isReference(_collectionType))
				{
					_collectionType.setReferenceTypeName(expression.getReferenceTypeName());
					_referenceTypeName = expression.getReferenceTypeName();
				}
			}
			else if (_collectionType.getType() != expression.getType().getType())
			{
				logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "validate");
				return false;
			}
			_collectionType.setIsArray(true);
		}
		logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "validate");
		return true;
	}

	public List getCollectionElements() 
	{
		logger.entering(sourceClass,Thread.currentThread().getName()+" "+ "getCollectionElements");

		logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "getCollectionElements");
		
		return _elements;
	}

	public int getCollectionSize() 
	{
		logger.entering(sourceClass,Thread.currentThread().getName()+" "+ "getCollectionSize");
		
		logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "getCollectionSize");
		
		return _size;
	}

	public boolean isArray() 
	{
		return this.getType().getIsArray();
	}

	public TypeInfo getType() 
	{
		if (_collectionType == null) {
           try
            {
                validate();	// Sets the collection type.
            }
            catch (SPLException e)
            {
                logger.severe(Thread.currentThread().getName()+" "+"Exception caught");
            }

		}
		return _collectionType;
	}

    public String toString()
    {
        logger.entering(sourceClass,Thread.currentThread().getName()+" "+ "toString");
        
        String str="[ ";
        Iterator ExpIt = _elements.iterator();
        while(ExpIt.hasNext())
        {
            Expression exp=(Expression)ExpIt.next();
            if(exp!=null)
            {
            	String expstr=exp.toString();
            	str+=expstr + " , ";
            }
        }
        str=str.substring(0, str.length()-3);
        str+=" ]";
        
        logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "toString");
       
        return str;
    }

	public String getReferenceTypeName() throws SPLException 
	{
		return _referenceTypeName;
	}
 
/*	
	public void setElements(List elements) {
		logger.entering(sourceClass, Thread.currentThread().getName() + " "
				+ "setElements");

		_elements = elements;
		logger.exiting(sourceClass, Thread.currentThread().getName() + " "
				+ "setElements");

	}

	public void setSize(int size) {
		logger.entering(sourceClass, Thread.currentThread().getName() + " "
				+ "setSize");

		this.size = size;
		logger.exiting(sourceClass, Thread.currentThread().getName() + " "
				+ "setSize");

	}

	public void setType(int expressionType) {
		logger.entering(sourceClass, Thread.currentThread().getName() + " "
				+ "setType");

		this.collectionType = expressionType;
		logger.exiting(sourceClass, Thread.currentThread().getName() + " "
				+ "setType");

	}
*/
}