/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.  
 */
//

/**
 * @author Prashant Baliga <prabalig@in.ibm.com>
 *
 */

package org.apache.imperius.spl.parser.expressions;

import java.util.Iterator;
import java.util.List;
import java.util.Vector;
import java.util.logging.Logger;

import org.apache.imperius.spl.core.Expression;
import org.apache.imperius.spl.core.TypeInfo;
import org.apache.imperius.spl.parser.exceptions.SPLException;
import org.apache.imperius.spl.parser.util.TypeInfoImpl;
import org.apache.imperius.util.Messages;
import org.apache.imperius.util.SPLLogger;


public abstract class TripleArgumentExpression implements Expression
{
    
    protected Expression _lhsExp;

	protected Expression _midExp;

	protected Expression _rhsExp;

	protected TypeInfo _dataType = new TypeInfoImpl();

	private static Logger logger = SPLLogger.getSPLLogger().getLogger();

	private final String sourceClass = "TripleArgumentExpression";
    
    public boolean isArray()
    {
        logger.entering(sourceClass, Thread.currentThread().getName() + " "
				+ "isArray");

		logger.exiting(sourceClass, Thread.currentThread().getName() + " "
				+ "isArray");

		return _dataType.getIsArray();
    }
    
    public TripleArgumentExpression(List exprList) throws SPLException
    {
        logger.entering(sourceClass, Thread.currentThread().getName() + " "
				+ "TripleArgumentExpression");

		if (exprList != null) 
		{
			if (exprList.size() == 3) 
			{
				Expression l = (Expression) exprList.get(0);
				Expression m = (Expression) exprList.get(1);
				Expression r = (Expression) exprList.get(2);
				
				if (l == null) 
				{
					logger.severe(Thread.currentThread().getName() + " "
							+ "left hand side expression passed in is null.");
					throw new IllegalArgumentException(Messages.getString(
							"SPL_PASSED_EXPRESSION_ERROR_MSG",
							new Object[] { "left" }));
				}

				if (m == null) 
				{
					logger.severe(Thread.currentThread().getName() + " "
							+ "middle expression passed in is null.");
					throw new IllegalArgumentException(Messages.getString(
							"SPL_PASSED_EXPRESSION_ERROR_MSG",
							new Object[] { "middle" }));
				}

				if (r == null) 
				{
					logger.severe(Thread.currentThread().getName() + " "
							+ "right hand side expression passed in is null.");
					throw new IllegalArgumentException(Messages.getString(
							"SPL_PASSED_EXPRESSION_ERROR_MSG",
							new Object[] { "right" }));
				}

				this._lhsExp = l;
				this._midExp = m;
				this._rhsExp = r;

			} 
			else 
			{
				logger.severe(Thread.currentThread().getName() + " "
						+ "number of parameters passed is not 3.");
				throw new IllegalArgumentException(Messages.getString(
						"SPL_NO_OF_ARGUMENTS_PASSED_ERROR_MSG", new Object[] {
								"3", "" + exprList.size() }));
			}
		} 
		else 
		{
			logger.severe(Thread.currentThread().getName() + " "
					+ "parameters passed are null.");

			throw new IllegalArgumentException(Messages.getString(
					"SPL_NO_OF_ARGUMENTS_PASSED_ERROR_MSG", new Object[] {
							"3", "empty"}));
		}

		logger.exiting(sourceClass, Thread.currentThread().getName() + " "
				+ "TripleArgumentExpression");
        
    }
    
    public abstract Object evaluate() throws SPLException;
    
    public TypeInfo getType()
    {
        logger.entering(sourceClass, Thread.currentThread().getName() + " "
				+ "getType");

		logger.exiting(sourceClass, Thread.currentThread().getName() + " "
				+ "getType");

		return _dataType;
    }
    
    public String toString()
    {
        logger.entering(sourceClass, Thread.currentThread().getName() + " "
				+ "toString");

		String className = this.getClass().getName();

		String str = className.substring(className.lastIndexOf(".") + 1,
				className.length())
				+ "( "
				+ this._lhsExp.toString()
				+ " , "
				+ this._midExp.toString()
				+ " , "
				+ this._rhsExp.toString()
				+ " )";

		logger.exiting(sourceClass, Thread.currentThread().getName() + " "
				+ "toString");

		return str;
    }
 
    
    public abstract boolean validate() throws SPLException;
    
    public String getReferenceTypeName() throws SPLException
    {
    	return null;
    }
    
    public Iterator getSubexpressions() {
    	Vector subexpressions = new Vector();
    	subexpressions.add(_lhsExp);
    	subexpressions.add(_midExp);
    	subexpressions.add(_rhsExp);
    	return subexpressions.iterator();
    }
    
    //xiping 03/06/09 to support collection analysis
    public Expression getLHSExpression() {
    	return _lhsExp;
    }

    public Expression getMIDExpression() {
    	return _midExp;
    }

    public Expression getRHSExpression() {
    	return _rhsExp;
    }

    public void replaceLHSExpression(Expression exp) {
    	_lhsExp = exp;
    }

    public void replaceMIDExpression(Expression exp) {
    	_midExp = exp;
    }

    public void replaceRHSExpression(Expression exp) {
    	_rhsExp = exp;
    }

}
