/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.  
 */
//

/**
 * @author Xiping Wang
 * @modified Neeraj Joshi
 *
 */
package org.apache.imperius.spl.parser.expressions.impl;

import java.util.List;
import java.util.logging.Logger;

import org.apache.imperius.spl.core.TypeConstants;
import org.apache.imperius.spl.core.TypeInfo;
import org.apache.imperius.spl.parser.exceptions.SPLException;
import org.apache.imperius.spl.parser.expressions.StringExpression;
import org.apache.imperius.spl.parser.expressions.TripleArgumentExpression;
import org.apache.imperius.spl.parser.util.TypeResolver;
import org.apache.imperius.util.Messages;
import org.apache.imperius.util.SPLLogger;


public class LeftSubstring extends TripleArgumentExpression implements
        StringExpression
{
    
    public static final String className = LeftSubstring.class.getName();
    
    private static Logger logger = SPLLogger.getSPLLogger().getLogger();
    private static final String sourceClass="LeftSubstring";
    
    
    
    public LeftSubstring(List exprList, boolean validateExpression)
            throws SPLException
    {
        super(exprList);
        logger.entering(sourceClass,Thread.currentThread().getName()+" "+ "LeftSubstring");

        if (validateExpression)
        {
            if (!validate())
            {
                logger.severe(Thread.currentThread().getName()+" "+"validation error: " + className
                        + " has wrong data type passed in.");
                
                throw new SPLException(Messages.getString(
    					"SPL_VALIDATION_ERROR_MSG", new Object[] { className }));
            }
        }
        logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "LeftSubstring");
        
    }
    
    public Object evaluate() throws SPLException
    {
        logger.entering(sourceClass,Thread.currentThread().getName()+" "+ "evaluate");

        try
        {
            String o1 = (String) _lhsExp.evaluate();
            Object o2 = _midExp.evaluate();
            String o3 = (String) _rhsExp.evaluate();
            
            boolean leftToRight;
            if (o3.equals("LeftToRight"))
            {
                leftToRight = true;
            }
            else if (o3.equals("RightToLeft"))
            {
                leftToRight = false;
            }
            else
            {
                logger.severe(
                "Illegal direction argument.");
                
                throw new IllegalArgumentException(Messages
						.getString("SPL_INVALID_DIRECTION_ARGUMENT_ERROR_MSG"));

            }
            
            if (o2 instanceof String)
            {
                logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "evaluate");
                String sub = _leftSubstring(o1, (String) o2, leftToRight); 
                return sub;
            }
            else
            {
                logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "evaluate");
                String sub = _leftSubstring(o1, ((Number) o2).intValue(), leftToRight); 
                return sub;
            }
        }
        catch (Exception e)
        {
            logger.severe(Thread.currentThread().getName()+" "+"evaluation error: " + e.toString());
            
            throw new SPLException(Messages.getString(
					"SPL_EVALUATION_ERROR_MSG", new Object[] { e
							.getLocalizedMessage() }));

        }
    }
    
    /**
     * Returns a specified LeftSubstring from a string. If
     * directionLeftToRight=true, numberChars is the number of characters to
     * return starting from the left. If the number is 2, the first two
     * characters of the string are returned; if the number is 5, the first five
     * characters are returned, and so on. If the number is negative, the entire
     * string is returned. If instead the directionLeftToRight=false,
     * numberChars is the number of characters to skip counting from right to
     * left. All the characters to the left of that number of characters are
     * returned. If the number is negative, the empty string is returned.<br>
     * Examples: LeftSubstring("AutonomicComputing", 4, LeftToRight) = "Auto".
     * LeftSubstring("AutonomicComputing", 4, RightToLeft) = "AutonomicCompu".
     * 
     * @param string
     * @param numberChars
     * @param directionLeftToRight
     * 
     * @return the resulting substring
     */
    private String _leftSubstring(String string, int numberChars,
            boolean directionLeftToRight)
    {
        logger.entering(sourceClass,Thread.currentThread().getName()+" "+ "leftSubstring");

        
        if (directionLeftToRight)
        {
            if (numberChars < 0)
            {
                logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "leftSubstring");
                
                return string;
            }
            else if (numberChars > string.length())
            {
                logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "leftSubstring");
                
                return string;
            }
            else
            {
                logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "leftSubstring");
                
                return string.substring(0, numberChars);
            }
        }
        else
        {
            if (numberChars < 0)
            {
                logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "leftSubstring");
                
                return "";
            }
            else if (numberChars > string.length())
            {
                logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "leftSubstring");
                
                return string;
            }
            else
            {
                logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "leftSubstring");
                
                return string.substring(0, string.length() - numberChars);
            }
        }
    }
    
    /**
     * Returns a specified LeftSubstring from a string. LeftSubstring returns
     * the characters from the String the left of the startString, not including
     * the startString characters. If directionLeftToRight=true, then it finds
     * startString by searching the String from left to right. If
     * directionLeftToRight=false, then it finds StartString by searching the
     * String from right to left. If the StartString is not found in the String,
     * then an empty string is returned.<br>
     * Examples: LeftSubstring("AutonomicComputing", "om", LeftToRight)="Auton"
     * LeftSubstring("AutonomicComputing", "om", RightToLeft)="AutonomicC"
     * 
     * @param string
     * @param startString
     * @param directionLeftToRight
     * 
     * @return the resulting substring
     */
    private String _leftSubstring(String string, String startString,
            boolean directionLeftToRight)
    {

        logger.entering(sourceClass,Thread.currentThread().getName()+" "+ "leftSubstring");

        int offsetpoint; // Position of the beginning of the startString
        
        if (directionLeftToRight)
        {
            offsetpoint = string.indexOf(startString);
            if (offsetpoint < 0)
            {
                // Did not find the string
                logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "leftSubstring");
                
                return "";
            }
        }
        else
        {
            offsetpoint = string.lastIndexOf(startString);
            if (offsetpoint < 0)
            {
                // Did not find the string
                logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "leftSubstring");
                
                return "";
            }
        }
        logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "leftSubstring");
        
        return string.substring(0, offsetpoint);
    }
    
    public boolean validate() throws SPLException
    {
        logger.entering(sourceClass,Thread.currentThread().getName()+" "+ "validate");

        TypeInfo lType = _lhsExp.getType();
        TypeInfo mType = _midExp.getType();
        TypeInfo rType = _rhsExp.getType();
        
        if (!lType.getIsArray() && !mType.getIsArray() && 
        		!rType.getIsArray() && TypeResolver.isString(lType) && 
        		(TypeResolver.isString(mType) || TypeResolver
                        .isNumeric(mType)) && TypeResolver.isString(rType))
        {
            _dataType.setType(TypeConstants.stringType);
            logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "validate");
            
            return true;
        }
        else
        {
            throw new SPLException(Messages.getString(
					"SPL_WRONG_OPERAND_TYPE_EXCEPTION1_MSG",
					new Object[] { lType + " , " + mType + " , " + rType }));
        }
    }
    
    public String toString()
    {
        logger.entering(sourceClass,Thread.currentThread().getName()+" "+ "toString");

        String str = "LeftSubstring("+this._lhsExp.toString() + "," + this._midExp.toString() + "," + this._rhsExp.toString() +")";
        	
        logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "toString");
       
        return str;
    }
    
}
