/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.  
 */
//

/**
 * @author Xiping Wang
 *
 */
package org.apache.imperius.spl.parser.expressions.impl;

import java.util.List;
import java.util.logging.Logger;

import org.apache.imperius.spl.core.TypeConstants;
import org.apache.imperius.spl.core.TypeInfo;
import org.apache.imperius.spl.parser.exceptions.SPLException;
import org.apache.imperius.spl.parser.expressions.DoubleArgumentExpression;
import org.apache.imperius.spl.parser.expressions.NumericExpression;
import org.apache.imperius.spl.parser.util.TypeResolver;
import org.apache.imperius.util.Messages;
import org.apache.imperius.util.SPLLogger;


public class Mod extends DoubleArgumentExpression implements NumericExpression
{
    
    public static final String className = Mod.class.getName();
    
    private static Logger logger = SPLLogger.getSPLLogger().getLogger();
    private static final String sourceClass="Mod";
    
    
    
    public Mod(List exprList, boolean validateExpression)
            throws SPLException
    {
        super(exprList);
        logger.entering(sourceClass,Thread.currentThread().getName()+" "+ "Mod");

        if (validateExpression)
        {
            if (!validate())
            {
                logger.severe(Thread.currentThread().getName()+" "+"validation error: " + className
                        + " has wrong data type passed in.");
                
                throw new SPLException(Messages.getString(
    					"SPL_VALIDATION_ERROR_MSG", new Object[] { className }));
            }
        }
        logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "Mod");
        
    }
    
    public Object evaluate() throws SPLException
    {
        logger.entering(sourceClass,Thread.currentThread().getName()+" "+ "evaluate");

        try
        {
            Number lhs = (Number) _lhsExp.evaluate();
            Number rhs = (Number) _rhsExp.evaluate();

            logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "evaluate");
         
            return remainder(lhs, rhs);
        }
        catch (Exception e)
        {
            logger.severe(Thread.currentThread().getName()+" "+"evaluation error: " + e.toString());
            
            throw new SPLException(Messages.getString(
					"SPL_EVALUATION_ERROR_MSG", new Object[] { e
							.getLocalizedMessage() }));

        }
    }
    
    public static Number remainder(Number o1, Number o2) throws SPLException
    {
        logger.entering(sourceClass,Thread.currentThread().getName()+" "+ "remainder");

        Number[] o = new Number[2];
        o[0] = o1;
        o[1] = o2;
        
        int resolvedType = TypeResolver.resolveType(o);
        
        if (resolvedType == TypeConstants.doubleType)
        {
            logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "remainder");
            
            return new Double(o1.doubleValue() % o2.doubleValue());
        }
        else if (resolvedType == TypeConstants.floatType)
        {
            logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "remainder");
            
            return new Float(o1.floatValue() % o2.floatValue());
        }
        else if (resolvedType == TypeConstants.longType)
        {
            logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "remainder");
            
            return new Long(o1.longValue() % o2.longValue());
        }
        else if (resolvedType == TypeConstants.intType)
        {
            logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "remainder");
            
            return new Integer(o1.intValue() % o2.intValue());
        }
        else if (resolvedType == TypeConstants.shortType)
        {
            logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "remainder");
            
            return new Short((short) (o1.shortValue() % o2.shortValue()));
        }
        else if (resolvedType == TypeConstants.byteType)
        {
            logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "remainder");
            
            return new Byte((byte) (o1.byteValue() % o2.byteValue()));
        }
        else
        {
            logger.severe(Thread.currentThread().getName()+" "+"Illegal argument type.");
            
            throw new SPLException(Messages.getString("SPL_WRONG_OPERAND_TYPE_EXCEPTION_MSG"));
        }
    }
    
    public boolean validate() throws SPLException
    {
        logger.entering(sourceClass,Thread.currentThread().getName()+" "+ "validate");

        TypeInfo lType = _lhsExp.getType();
        TypeInfo rType = _rhsExp.getType();
        
        // dataType = TypeConstants.numericType;
        if (TypeResolver.isNumeric(lType) && TypeResolver.isNumeric(rType))
        {
            _dataType = TypeResolver
                    .binaryNumericPromotionResolver(lType, rType);
            this._dataType.setIsArray(false);
            logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "validate");
            
            return true;
        }

        logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "validate");
     
        return false;
    }
    
    public String toString()
    {
        logger.entering(sourceClass,Thread.currentThread().getName()+" "+ "toString");
        
        String str=this._lhsExp.toString() + " % " + this._rhsExp.toString();
        
        logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "toString");
       
        return str;
    }
    
}
