/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.  
 */
//

/**
 * @author Xiping Wang
 *
 */
package org.apache.imperius.spl.parser.expressions.impl;

import java.util.List;
import java.util.logging.Logger;

import org.apache.imperius.spl.core.TypeConstants;
import org.apache.imperius.spl.core.TypeInfo;
import org.apache.imperius.spl.parser.exceptions.SPLException;
import org.apache.imperius.spl.parser.expressions.StringExpression;
import org.apache.imperius.spl.parser.expressions.TripleArgumentExpression;
import org.apache.imperius.spl.parser.util.TypeResolver;
import org.apache.imperius.util.Messages;
import org.apache.imperius.util.SPLLogger;


public class ReplaceSubstring extends TripleArgumentExpression implements
        StringExpression
{
    
    public static final String className = ReplaceSubstring.class.getName();
    
    private static Logger logger = SPLLogger.getSPLLogger().getLogger();
    private static final String sourceClass="ReplaceSubstring";
    
    
    
    public ReplaceSubstring(List exprList, boolean validateExpression)
            throws SPLException
    {
        super(exprList);
        logger.entering(sourceClass,Thread.currentThread().getName()+" "+ "ReplaceSubstring");

        if (validateExpression)
        {
            if (!validate())
            {
                logger.severe(Thread.currentThread().getName()+" "+"validation error: " + className
                        + " has wrong data type passed in.");
                
                throw new SPLException(Messages.getString(
    					"SPL_VALIDATION_ERROR_MSG", new Object[] { className }));
            }
        }
        logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "ReplaceSubstring");
        
    }
    
    public Object evaluate() throws SPLException
    {
        logger.entering(sourceClass,Thread.currentThread().getName()+" "+ "evaluate");

        try
        {
            String o1 = (String) _lhsExp.evaluate();
            String o2 = (String) _midExp.evaluate();
            String o3 = (String) _rhsExp.evaluate();

            
            String result = _replaceSubstring(o1, o2, o3);
            logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "evaluate" + result);
            return result;
        }
        catch (Exception e)
        {
            logger.severe(Thread.currentThread().getName()+" "+"evaluation error: " + e.toString());
            
            throw new SPLException(Messages.getString(
					"SPL_EVALUATION_ERROR_MSG", new Object[] { e
							.getLocalizedMessage() }));

        }
    }
    
    /**
     * Returns a string resulting from replacing all instances of the
     * fromSubstring with toSubstring in a given string.
     * <p>
     * 
     * @param baseString
     *            base string
     * @param fromSubstring
     *            substring to replace
     * @param toSubstring
     *            substring to put in place of fromSubtring
     * @return the new string
     */
    private String _replaceSubstring(String baseString,
            String fromSubstring, String toSubstring)
    {
        logger.entering(sourceClass,Thread.currentThread().getName()+" "+ "replaceSubstring");

        String result = "";
        int fromLength = fromSubstring.length();
        
        int pos = baseString.indexOf(fromSubstring);
        int lastpos = 0;
        while (pos >= 0)
        {
            result += baseString.substring(lastpos, pos) + toSubstring;
            lastpos = pos + fromLength;
            pos = baseString.indexOf(fromSubstring, lastpos);
        }
        if (lastpos < baseString.length())
        {
            result += baseString.substring(lastpos, baseString.length());
        }
        logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "replaceSubstring");
        
        return result;
        
    }
    
    public boolean validate() throws SPLException
    {
        logger.entering(sourceClass,Thread.currentThread().getName()+" "+ "validate");

        logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "validate");
     
        TypeInfo lType = _lhsExp.getType();
        TypeInfo mType = _midExp.getType();
        TypeInfo rType = _rhsExp.getType();
        
        if (!lType.getIsArray() && !mType.getIsArray() && !rType.getIsArray() &&
        		TypeResolver.isString(lType) && TypeResolver.isString(mType)
                && TypeResolver.isString(rType))
        {
            _dataType.setType(TypeConstants.stringType);
            logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "validate");
            
            return true;
        }
        logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "validate");
        
        
        return false;
    }
    
    public String toString()
    {
        logger.entering(sourceClass,Thread.currentThread().getName()+" "+ "toString");
        
        String str="ReplaceSubstring("+this._lhsExp.toString()+","+ this._midExp.toString() + ","+this._rhsExp.toString() + ")";
        
        logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "toString");
       
        return str;
    }
    
}
