/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.  
 */
//

/**
 * @author Xiping Wang
 * @modified Neeraj Joshi
 *
 */
package org.apache.imperius.spl.parser.expressions.impl;

import java.util.List;
import java.util.logging.Logger;

import org.apache.imperius.spl.core.TypeConstants;
import org.apache.imperius.spl.core.TypeInfo;
import org.apache.imperius.spl.parser.exceptions.IllegalExpressionTypeException;
import org.apache.imperius.spl.parser.exceptions.SPLException;
import org.apache.imperius.spl.parser.expressions.NumericExpression;
import org.apache.imperius.spl.parser.expressions.SingleArgumentExpression;
import org.apache.imperius.spl.parser.util.TypeResolver;
import org.apache.imperius.util.Messages;
import org.apache.imperius.util.SPLLogger;


public class Round extends SingleArgumentExpression implements
        NumericExpression
{
    
    public static final String className = Round.class.getName();
    
    private static Logger logger = SPLLogger.getSPLLogger().getLogger();
    private static final String sourceClass="Round";
    
    
    
    public Round(List exprList, boolean validateExpression)
            throws SPLException
    {
        super(exprList);
        logger.entering(sourceClass,Thread.currentThread().getName()+" "+ "Round");

        if (validateExpression)
        {
            if (!validate())
            {
                logger.severe(Thread.currentThread().getName()+" "+"validation error: " + className
                        + " has wrong data type passed in.");
                
                throw new SPLException(Messages.getString(
    					"SPL_VALIDATION_ERROR_MSG", new Object[] { className }));
            }
        }
        logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "Round");
        
    }
    
    public Object evaluate() throws SPLException
    {
        logger.entering(sourceClass,Thread.currentThread().getName()+" "+ "evaluate");

        try
        {
        	Number result = _round((Number) _exp.evaluate()); 
            logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "evaluate");
            
            return result;
        }
        catch (Exception e)
        {
            logger.severe(Thread.currentThread().getName()+" "+"evaluation error: " + e.toString());
            
            throw new SPLException(Messages.getString(
					"SPL_EVALUATION_ERROR_MSG", new Object[] { e
							.getLocalizedMessage() }));

        }
    }
    
    private Number _round(Number o1) throws IllegalExpressionTypeException
    {
        logger.entering(sourceClass,Thread.currentThread().getName()+" "+ "round");

        Number[] o = new Number[1];
        o[0] = o1;
        
        int resolvedType = TypeResolver.resolveType(o);
        
        if (resolvedType == TypeConstants.doubleType)
        {
            long roundVal = Math.round(o1.doubleValue());
            
            logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "round");
            
            return new Long(roundVal);
        }
        else if (resolvedType == TypeConstants.floatType)
        {
            int roundVal = Math.round(o1.floatValue());
            logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "round");
            
            return new Integer(roundVal);
        }
        else
        {
            logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "round");
            
            return new Integer(o1.intValue());
        }
    }
    
    public boolean validate() throws SPLException
    {
        logger.entering(sourceClass,Thread.currentThread().getName()+" "+ "validate");

        TypeInfo eType = _exp.getType();
        
        // dataType = TypeConstants.numericType;
        if (TypeResolver.isNumeric(eType))
        {
            int tp = eType.getType();
            if (tp == TypeConstants.doubleType)
            {
                _dataType.setType(TypeConstants.longType);
                
            }
            else if (tp == TypeConstants.floatType)
            {
                _dataType.setType(TypeConstants.intType);
                
            }
            else
            {
                _dataType.setType(TypeConstants.intType);
            }
            logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "validate");
            
            return true;
        }

        logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "validate");
     
        return false;
    }
    
    public String toString()
    {
        logger.entering(sourceClass,Thread.currentThread().getName()+" "+ "toString");
        
        String str="ROUND("+this._exp.toString()+")";
        
        logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "toString");
       
        return str;
    }
    
}
