/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.  
 */
//

/**
 * @author Neeraj Joshi
 *
 */
package org.apache.imperius.spl.parser.expressions.impl;

import java.util.List;
import java.util.logging.Logger;

import org.apache.imperius.spl.core.TypeConstants;
import org.apache.imperius.spl.core.TypeInfo;
import org.apache.imperius.spl.parser.exceptions.SPLException;
import org.apache.imperius.spl.parser.expressions.DoubleArgumentExpression;
import org.apache.imperius.spl.parser.expressions.NumericExpression;
import org.apache.imperius.spl.parser.util.TypeResolver;
import org.apache.imperius.util.Messages;
import org.apache.imperius.util.SPLLogger;


public class Subtraction extends DoubleArgumentExpression implements
        NumericExpression
{
    
    public static final String className = Subtraction.class.getName();
    
    private static Logger logger = SPLLogger.getSPLLogger().getLogger();
    private static final String sourceClass="Subtraction";
    
    
    
    public Subtraction(List exprList, boolean validateExpression)
            throws SPLException
    {
        super(exprList);
        logger.entering(sourceClass,Thread.currentThread().getName()+" "+ "Subtraction");

        if (validateExpression)
        {
            if (!validate())
            {
                logger.severe(Thread.currentThread().getName()+" "+"validation error: " + className
                        + " has wrong data type passed in.");
                
                throw new SPLException(Messages.getString(
    					"SPL_VALIDATION_ERROR_MSG", new Object[] { sourceClass }));
            }
        }
        logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "Subtraction");
        
    }
    
    public Object evaluate() throws SPLException
    {
        logger.entering(sourceClass,Thread.currentThread().getName()+" "+ "evaluate");
        try
        {
            Number lhs = (Number) _lhsExp.evaluate();
            Number rhs = (Number) _rhsExp.evaluate();

            logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "evaluate");
         
          
            return minus(lhs, rhs);
        }
        catch (Exception e)
        {
            logger.severe(Thread.currentThread().getName()+" "+"evaluation error: " + e.toString());
            
            
            throw new SPLException(Messages.getString(
					"SPL_EVALUATION_ERROR_MSG", new Object[] { e
							.getLocalizedMessage() }));

        }
    }
    public static Number minus(Number o1) throws SPLException
    {
        logger.entering(sourceClass,Thread.currentThread().getName()+" "+ "minus(Number)");
        
        Number[] o = new Number[1];
        o[0] = o1;
        
        int resolvedType = TypeResolver.resolveType(o);
        
        if (resolvedType == TypeConstants.doubleType)
        {
            double result = o1.doubleValue() * (-1);
            logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "minus");
            
            return new Double(result);
        }
        else if (resolvedType == TypeConstants.floatType)
        {
            float result = o1.floatValue() * (-1);
            logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "minus");
            
            return new Float(result);
        }
        else if (resolvedType == TypeConstants.longType)
        {
            long result = o1.longValue() * (-1);
            logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "minus");
            
            return new Long(result);
        }
        else
        {
            int result = o1.intValue() * (-1);
            logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "minus");
            
            return new Integer(result);
        }
    }
    
    public static Number minus(Number o1, Number o2) throws SPLException
    {
        logger.entering(sourceClass,Thread.currentThread().getName()+" "+ "minus");

        
        Number[] o = new Number[2];
        o[0] = o1;
        o[1] = o2;
        
        int resolvedType = TypeResolver.resolveType(o);
        
        if (resolvedType == TypeConstants.doubleType)
        {
            double result = o1.doubleValue() - o2.doubleValue();
            logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "minus");
            
            return new Double(result);
        }
        else if (resolvedType == TypeConstants.floatType)
        {
            float result = o1.floatValue() - o2.floatValue();
            logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "minus");
            
            return new Float(result);
        }
        else if (resolvedType == TypeConstants.longType)
        {
            long result = o1.longValue() - o2.longValue();
            logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "minus");
            
            return new Long(result);
        }
        else
        {
            int result = o1.intValue() - o2.intValue();
            logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "minus");
            
            return new Integer(result);
        }
    }
    
    public boolean validate() throws SPLException
    {
        logger.entering(sourceClass,Thread.currentThread().getName()+" "+ "validate");

        TypeInfo lType = _lhsExp.getType();
        TypeInfo rType = _rhsExp.getType();
        
        // dataType = TypeConstants.numericType;
        if (!lType.getIsArray() && !rType.getIsArray() &&
        		TypeResolver.isNumeric(lType) && TypeResolver.isNumeric(rType))
        {
            _dataType = TypeResolver
                    .binaryNumericPromotionResolver(lType, rType);
            logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "validate");
            
            return true;
        }

        logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "validate");
     
        return false;
    }
    
    public String toString()
    {
        logger.entering(sourceClass,Thread.currentThread().getName()+" "+ "toString");
        String retStr= this._lhsExp.toString() + "-" + this._rhsExp.toString();
        logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "toString");
       
        return retStr;
    }
    
}
