/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.  
 */
//

/**
 * @author Prashant Baliga <prabalig@in.ibm.com>
 *
 */
package org.apache.imperius.spl.parser.expressions.impl;

import java.util.List;
import java.util.logging.Logger;

import org.apache.imperius.spl.core.TypeConstants;
import org.apache.imperius.spl.core.TypeInfo;
import org.apache.imperius.spl.parser.exceptions.SPLException;
import org.apache.imperius.spl.parser.expressions.BooleanExpression;
import org.apache.imperius.spl.parser.expressions.SingleArgumentExpression;
import org.apache.imperius.spl.parser.util.TypeResolver;
import org.apache.imperius.util.Messages;
import org.apache.imperius.util.SPLLogger;


public class ToBoolean extends SingleArgumentExpression implements
        BooleanExpression
{
    
    public static final String className = ToBoolean.class.getName();
    
    private static Logger logger = SPLLogger.getSPLLogger().getLogger();
    private static final String sourceClass="ToBoolean";
    
    
    
    public ToBoolean(List exprList, boolean validateExpression)
            throws SPLException
    {
        super(exprList);
        logger.entering(sourceClass,Thread.currentThread().getName()+" "+ "ToBoolean");

        if (validateExpression)
        {
            if (!validate())
            {
                logger.severe(Thread.currentThread().getName()+" "+"validation error: " + className
                        + " has wrong data type passed in.");
                throw new SPLException(Messages.getString(
    					"SPL_VALIDATION_ERROR_MSG", new Object[] { sourceClass }));
            }
        }
        logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "ToBoolean");
        
    }
    
    public Object evaluate() throws SPLException
    {
        logger.entering(sourceClass,Thread.currentThread().getName()+" "+ "evaluate");

        try
        {
            Object obj = _exp.evaluate();
           //System.out.println("after evaluation :" + obj.toString());
            
            if (_dataType.getType() == TypeConstants.doubleType)
            {
                // Necessary because double works differently
                Double number = (Double) obj;
                if (number.doubleValue() > 0 || number.doubleValue() < 0)
                {
                    logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "evaluate");
                    
                    return new Boolean(true);
                }
                else
                {
                    logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "evaluate");
                    
                    return new Boolean(false);
                }
            }
            else if (_dataType.getType() == TypeConstants.floatType)
            {
                // Necessary because float works differently
                Float number = (Float) obj;
                if (number.floatValue() > 0 || number.floatValue() < 0)
                {
                    logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "evaluate");
                    
                    return new Boolean(true);
                }
                else
                {
                    logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "evaluate");
                    
                    return new Boolean(false);
                }
            }
            else if (obj instanceof Number)
            {
                if (((Number) obj).longValue() == 0){

                    logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "evaluate");
                    return new Boolean(false);
                 
                }
                 
                else
                {
                    logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "evaluate");
                    
                    return new Boolean(true);
                }
            }
            else
            {
                logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "evaluate");
                
                // must be string type
                return new Boolean((String) obj);
            }
        }
        catch (Exception e)
        {
            logger.severe(Thread.currentThread().getName()+" "+"evaluation error: " + e.toString());
            
            throw new SPLException(Messages.getString(
					"SPL_EVALUATION_ERROR_MSG", new Object[] { e
							.getLocalizedMessage() }));

        }
    }
    
    public boolean validate() throws SPLException
    {
        logger.entering(sourceClass,Thread.currentThread().getName()+" "+ "validate");

        TypeInfo eType = _exp.getType();
       //System.out.println("validating : " + _exp.getType());
        
        if (!eType.getIsArray() && 
        		TypeResolver.isNumeric(eType) || TypeResolver.isString(eType))
        {
            _dataType.setType(TypeConstants.booleanType);
            logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "validate");
            
            return true;
        }
        else
        {
           //System.out.println("validation failure: " + _exp.toString() + _exp.getClass());
            logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "validate");
            
            return false;
        }
    }
    
    public String toString()
    {
        logger.entering(sourceClass,Thread.currentThread().getName()+" "+ "toString");
        String retStr= "toBoolean("+this._exp.toString() + ")";
        logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "toString");
       
        return retStr;
    }
    
}
