/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.  
 */
package org.apache.imperius.spl.parser.statements;

import java.util.ArrayList;
import java.util.List;

import org.apache.imperius.spl.parser.statements.impl.SPLPolicy;


/**
 * Used by {@link SPLPolicy} and its cohorts to capture the results of a policy evaluation.
 * This can hold the results of nested policies (i.e. policy groups) and multiple policy statements.
 * EventResults objects which contain the results of policy group results are appended and are 
 * considered nested results. 
 * {@link PolicyStatementResults} are appended for each policy statement (..., condition, decision set).
 * Statement and group results may be retrieved from an instance via the {@link #getPolicyStatementResults()}
 * and {@link #getNestedResults()} methods.  
 *
 * @author dawood
 *
 */
public class EvaluationResults {
	
	protected int statusCode = SPLPolicy.POLICY_NOT_EVALUATED;
	protected List statementResults = null;
	protected List evaluationResults = null;
	
	/**
	 * Create an result indicating that it has not been evaluated.
	 */
	public EvaluationResults() {	}
	

	/**
	 * Create the result to include the given statement result and its status code.
	 * @param psr
	 */
	public EvaluationResults(PolicyStatementResult psr) {
		statementResults = new ArrayList(1);
		statementResults.add(psr);
		addNewStatusCode(psr.statusCode);
	}

	/**
	 * Add the given result to the list of nested results
	 * The status code from the given result is merged with the current status code.
	 * @param er
	 */
	public void appendResult(EvaluationResults er) {
		if (evaluationResults == null)
			evaluationResults = new ArrayList(1);
		evaluationResults.add(er);
		addNewStatusCode(er.statusCode);
	}
	
	/**
	 * Add the given result to the list of statement results
	 * The status code from the given result is merged with the current status code.
	 * @param psr
	 */
	public void appendResult(PolicyStatementResult psr) {
		if (statementResults == null)
			statementResults = new ArrayList(1);
		statementResults.add(psr);
		addNewStatusCode(psr.statusCode);
	}

	/**
	 * Get the current status code for this result.
	 * The current status code is a merger of status codes from all the
	 * statement and evaluation results.  
	 * @return
	 */
	public int getStatusCode() { return statusCode; }
	
	/**
	 * Gets the list of statement results in this instance.
	 * Nested statement results are not returned by this method.
	 * @return null if not statement results
	 */
	public List getPolicyStatementResults() { return statementResults; }
	
	/**
	 * Gets the list of nested evaluation results.
	 * Results nested within immediately nested results are not returned in the list.
	 * @return null if no nested results are present.
	 */
	public List getNestedResults() { return evaluationResults; }
	
	private void addNewStatusCode(int newStatus) {
		switch (newStatus) {
			case SPLPolicy.POLICY_EVALUATION_FAILED:
				statusCode = SPLPolicy.POLICY_EVALUATION_FAILED;
				break;
			case SPLPolicy.POLICY_EVALUATED_SUCCESSFULLY:
				switch (statusCode) {
					case SPLPolicy.POLICY_EVALUATION_FAILED:			break;
					case SPLPolicy.POLICY_EVALUATED_SUCCESSFULLY:		break;
					case SPLPolicy.POLICY_NOT_EVALUATED: statusCode = SPLPolicy.POLICY_EVALUATED_SUCCESSFULLY;
				}
				break;
			case SPLPolicy.POLICY_NOT_EVALUATED:
				break;
			default:
				throw new RuntimeException("Illegal status " + newStatus);
				
		}

	}


}
