/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.  
 */
package org.apache.imperius.spl.parser.statements;

/**
 * Holds the information about the evaluation of a single policy statement.
 * This includes:
 * <ul>
 * <li> status code - one of 
 *    <ul>
 *    <li> SPLPolicy.POLICY_NOT_EVALUATED
 *    <li> SPLPolicy.POLICY_EVALUATED_SUCCESSFULLY
 *    <li> SPLPolicy.POLICY_EVALUATION_FAILED
 *    </ul>
 * <li> condition result - a Boolean that is null if there was not condition
 * in the policy, otherwise the value of the condition statement.
 * <li> decision result - a Boolean that is not null and contains the
 * success or failure of the execution of the decision.
 * </ul>
 * @author dawood
 *
 */
public class PolicyStatementResult {

	protected int statusCode;
	protected Boolean conditionResult = null;
	protected Boolean decisionResult = null;
	
	/**
	 * Create a result for a statement that has a condition.
	 * @param result - overall status of evaluation
	 * @param conditionResult - value of the condition statement
	 * @param decisionResult - success status of decision evaluation
	 */
	public PolicyStatementResult(int result, Boolean conditionResult,	Boolean decisionResult) {
		this.statusCode = result;
		this.conditionResult = conditionResult;
		this.decisionResult = decisionResult;
	}

	/**
	 * Create a result for a statement that does NOT have a condition.
	 * @param result - overall status of evaluation
	 * @param decisionResult - success status of decision evaluation
	 */
	public PolicyStatementResult(int result, Boolean decisionResult) {
		this(result,null, decisionResult);
	}
	
	/**
	 * Get the overall status of the evaluation.
	 * @return
	 */
	public int getStatusCode() { return statusCode; }
	
	/**
	 * Get the result of the condition evaluation if one existed.
	 * @return true or false or null if no condition existed in policy.
	 */
	public Boolean getConditionResult() { return conditionResult; }
	
	/**
	 * Get the result of the decision.
	 * @return true, false.
	 */
	public Boolean getDecisionResult() { return decisionResult; }

}
