/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.  
 */
//

/**
 * @author Neeraj Joshi <jneeraj@us.ibm.com>
 *
 */

package org.apache.imperius.spl.parser.util;



import java.util.List;

import org.apache.imperius.spl.core.Expression;
import org.apache.imperius.spl.core.TypeConstants;
import org.apache.imperius.spl.core.TypeInfo;
import org.apache.imperius.spl.parser.exceptions.IllegalExpressionTypeException;
import org.apache.imperius.spl.parser.exceptions.IllegalParameterTypeException;
import org.apache.imperius.spl.parser.exceptions.SPLException;
import org.apache.imperius.spl.parser.exceptions.TypesNotAssignableException;
import org.apache.imperius.util.Messages;




/**
 * Provides Number Expression Type resolution.
 * 
 */
public class TypeResolver
{
    
    public static final String OBJECT_STRING = "java.lang.Object";

//    private static final String sourceClass="TypeConstants";
    
    
    
    /*
     * public static final byte BYTE = 0; public static final byte SHORT = 1;
     * public static final byte INTEGER = 2; public static final byte LONG = 3;
     * public static final byte FLOAT = 4; public static final byte DOUBLE = 5;
     */

    /**
     * Determines the numeric type that should be used to interpret a collection
     * of <code>Number</code> operands. Throws a
     * <code>ClassCastException</code> if the operands collection contains
     * objects that are not of type <code>Number</code>.
     * 
     * 
     * @param operands
     *            an array of <code>Number</code> operands
     * @return a byte code corresponding to the resolved type
     * @throws ClassCastException
     *             if the collection does not contain <code>Number</code>
     *             operands[i]s
     * @throws IllegalExpressionTypeException
     */
    public static int resolveType(Number[] operands)
            throws IllegalExpressionTypeException
    {
        
        
        int resolvedType = TypeConstants.byteType;
        
        for (int i = 0; i < operands.length; i++)
        {
            if (operands[i] instanceof Double)
            {
                resolvedType = TypeConstants.doubleType;
                break;
            }
            else if (operands[i] instanceof Float)
            {
                if (resolvedType < TypeConstants.floatType)
                    resolvedType = TypeConstants.floatType;
            }
            else if (operands[i] instanceof Long)
            {
                if (resolvedType < TypeConstants.longType)
                    resolvedType = TypeConstants.longType;
            }
            else if (operands[i] instanceof Integer)
            {
                if (resolvedType < TypeConstants.intType)
                    resolvedType = TypeConstants.intType;
            }
            else if (operands[i] instanceof Short)
            {
                if (resolvedType < TypeConstants.shortType)
                    resolvedType = TypeConstants.shortType;
            }
        }

        
   
        return resolvedType;
    }
    
    /**
     * Determines the numeric type that should be used to interpret the Object
     * operands of a binary expression accoring to the Java Numeric Promotion
     * rules.
     * <p>
     * 
     * For reference: 2.6.9 Numeric Promotion Numeric promotion is applied to
     * the operands of an arithmetic operator. Numeric promotion contexts allow
     * the use of an identity conversion (Section 2.6.1) or a widening primitive
     * conversion (Section 2.6.2). Numeric promotions are used to convert the
     * operands of a numeric operator to a common type where an operation can be
     * performed. The two kinds of numeric promotion are unary numeric promotion
     * and binary numeric promotion. The analogous conversions in C are called
     * "the usual unary conversions" and "the usual binary conversions." Numeric
     * promotion is not a general feature of Java, but rather a property of the
     * specific definitions of built-in operators. An operator that applies
     * unary numeric promotion to a single operand of numeric type converts an
     * operand of type byte, short, or char to int, and otherwise leaves the
     * operand alone. The operands of the shift operators are promoted
     * independently using unary numeric promotions.
     * <p>
     * 
     * When an operator applies binary numeric promotion to a pair of numeric
     * operands, the following rules apply, in order, using widening conversion
     * (Section 2.6.2) to convert operands as necessary: If either operand is of
     * type double, the other is converted to double. Otherwise, if either
     * operand is of type float, the other is converted to float. Otherwise, if
     * either operand is of type long, the other is converted to long.
     * Otherwise, both operands are converted to type int.
     * <p>
     * Throws a <code>NullPointerException</code> if either operand is null.
     * 
     * @param leftOperand
     *            left operand of a binary expression
     * @param rightOperand
     *            right operand of a binary expression
     * @return a byte code corresponding to the resolved type
     * @throws SPLException 
     * @throws java.lang.NullPointerException
     *             if there is null operand
     */
    public static TypeInfoImpl binaryNumericPromotionResolver(TypeInfo ltOperand,
            TypeInfo rtOperand) throws SPLException
    {
        int leftOperand = ltOperand.getType();
        int rightOperand = rtOperand.getType();
        
        
        if (leftOperand == TypeConstants.doubleType
                || rightOperand == TypeConstants.doubleType)
        {
        
            
            return new TypeInfoImpl(TypeConstants.doubleType);
        }
        else if (leftOperand == TypeConstants.floatType
                || rightOperand == TypeConstants.floatType)
        {
        
            
        	return new TypeInfoImpl(TypeConstants.floatType);
        }
        else if (leftOperand == TypeConstants.longType
                || rightOperand == TypeConstants.longType)
        {
        
            
        	return new TypeInfoImpl(TypeConstants.longType);
        }
        else
        {
           
            
            return new TypeInfoImpl(TypeConstants.intType,null,false);
        }
        
    }
    
    /**
     * Determines the numeric type that should be used to interpret the Object
     * operand of a unary expression accoring to the Java Numeric Promotion
     * rules.
     * <p>
     * 
     * For reference: 2.6.9 Numeric Promotion Numeric promotion is applied to
     * the operands of an arithmetic operator. Numeric promotion contexts allow
     * the use of an identity conversion (Section 2.6.1) or a widening primitive
     * conversion (Section 2.6.2). Numeric promotions are used to convert the
     * operands of a numeric operator to a common type where an operation can be
     * performed. The two kinds of numeric promotion are unary numeric promotion
     * and binary numeric promotion. The analogous conversions in C are called
     * "the usual unary conversions" and "the usual binary conversions." Numeric
     * promotion is not a general feature of Java, but rather a property of the
     * specific definitions of built-in operators. An operator that applies
     * unary numeric promotion to a single operand of numeric type converts an
     * operand of type byte, short, or char to int, and otherwise leaves the
     * operand alone. The operands of the shift operators are promoted
     * independently using unary numeric promotions.
     * <p>
     * 
     * When an operator applies binary numeric promotion to a pair of numeric
     * operands, the following rules apply, in order, using widening conversion
     * (Section 2.6.2) to convert operands as necessary: If either operand is of
     * type double, the other is converted to double. Otherwise, if either
     * operand is of type float, the other is converted to float. Otherwise, if
     * either operand is of type long, the other is converted to long.
     * Otherwise, both operands are converted to type int.
     * <p>
     * Throws a <code>NullPointerException</code> if either operand is null.
     * 
     * @param operand
     *            operand of a unary expression
     * @return a byte code corresponding to the resolved type
     * @throws TypesNotAssignableException
     * @throws java.lang.NullPointerException
     *             if the operand is null
     * 
     * public static byte unaryNumericPromotionResolver(Number operand) throws
     * NullPointerException {
     * 
     * if (operand instanceof Double) { return DOUBLE; }
     * 
     * if (operand instanceof Float) { return FLOAT; }
     * 
     * if(operand instanceof Long ) { return LONG; } // short byte char get
     * converted to integer return INTEGER; h}
     */
    
    public static boolean isTypeAssignableForEquality(TypeInfo leftType,
            TypeInfo rightType)
    {
        
        
        boolean res = false;
        //System.out.println("isTypeAssignableForEquality");
        if (isNumeric(leftType) && isNumeric(rightType))
        {
            res = true;
        }
        else if (isBoolean(leftType) && isBoolean(rightType))
        {
            res = true;
        }
        else if (isString(leftType) && isString(rightType))
        {
            res = true;
        }
        else if (isCalendar(leftType) && isCalendar(rightType))
        {
            res = true;
        }
        else //if (isReference(leftType) && isReference(rightType))
        {
        	if(isReference(leftType))
        	{
//        		if(OBJECT_STRING.equalsIgnoreCase(leftType.getReferenceTypeName()) && isString(rightType))
//        		{
//        			res = true;
//        		} 
        		if (isString(rightType) && isAssignableFromString(leftType)) 
        		{
        			res = true;
        		}
        		
        		// xiping's test 09-24-09
        		else if (isNumeric(rightType) && isAssignableFromNumeric(leftType)) {
        			res = true;
        		}
        		//
        		else if(isReference(rightType))
        		{
        			res = true;
        		}
        	}
        }

        
   
        return res;
    }
    
    private static boolean isAssignableFromString(TypeInfo type) {
		String typeName = type.getReferenceTypeName();
		if (typeName != null) {
			try {
				Class clazz = type.getClass().forName(typeName);
				return clazz.isAssignableFrom(String.class);
			} catch (ClassNotFoundException e) {
				;
			}
		}
		return false;
		
	}    

    // xiping 09-24-09
    private static boolean isAssignableFromNumeric(TypeInfo type) {
		String typeName = type.getReferenceTypeName();
		if (typeName != null) {
			try {
				Class clazz = type.getClass().forName(typeName);
				return clazz.isAssignableFrom(Byte.class) ||
					clazz.isAssignableFrom(Short.class) ||
					clazz.isAssignableFrom(Integer.class) ||
					clazz.isAssignableFrom(Long.class) ||
					clazz.isAssignableFrom(Float.class) ||
					clazz.isAssignableFrom(Double.class);
			} catch (ClassNotFoundException e) {
				;
			}
		}
		return false;
		
	}    

    //
    
    public static boolean isTypeAssignableForRelation(TypeInfo leftType,
            TypeInfo rightType)
    {
      
        
        boolean res = false;
        if (isNumeric(leftType) && isNumeric(rightType))
        {
            res = true;
        }
        else if (isString(leftType) && isString(rightType))
        {
            res = true;
        }
        else if (isCalendar(leftType) && isCalendar(rightType))
        {
            res = true;
        }

      
   
        return res;
    }
    
    public static TypeInfo resolveType(TypeInfo leftType, TypeInfo rightType)
            throws SPLException
    {
       
   
        // TBD
        return binaryNumericPromotionResolver(leftType, rightType);
        
    }
    
    public static boolean isNumeric(TypeInfo type)
    {
       int tp = type.getType();
        
        if (tp == TypeConstants.byteType || tp == TypeConstants.shortType
                || tp == TypeConstants.intType || tp == TypeConstants.longType
                || tp == TypeConstants.floatType
                || tp == TypeConstants.doubleType)
        {
       
            
            return true;
        }

       
   
        return false;
    }
    
    public static boolean isBoolean(TypeInfo type)
    {
       int tp = type.getType();
        
        if (tp == TypeConstants.booleanType)
        {
          
            
            return true;
        }

     
   
        return false;
    }
    
    public static boolean isReference(TypeInfo type) 
    {
		int tp = type.getType();
		if (tp == TypeConstants.referenceType) 
		{
			return true;
		}
		return false;
	}
    
    
    public static boolean isString(TypeInfo tp)
    {
    	int type = tp.getType();
        
        if (type == TypeConstants.stringType)
        {
          
            
            return true;
        }

        
   
        return false;
    }
    
    public static boolean isCalendar(TypeInfo tp)
    {
    	int type = tp.getType();
        
        if (type == TypeConstants.dateTime)
        {
   
            
            return true;
        }

   
   
        return false;
    }
    
    /*public static boolean validateActualParameterTypes(ArrayList formalParams,
            ArrayList passedParams) throws SPLException
    {

     
        
        if (formalParams.size() == passedParams.size())
        {
            for (int i = 0; i < formalParams.size(); i++)
            {
                Integer fParam = (Integer) formalParams.get(i);
                Expression pParam = (Expression) passedParams.get(i);
                
                if (!TypeResolver.isTypeAssignableForEquality(
                        fParam.intValue(), pParam.getType()))
                {
                   //System.out.println("type mismatch fParam.intValue()="+fParam.intValue()+" pParam.getType()"+pParam.getType());
                  
                    throw new IllegalParameterTypeException("param mismatch");
                }
                
            }
        }
      
        
        return true;
    }*/
    
    public static TypeInfo unaryNumericPromotionResolver(TypeInfo te)
            throws NullPointerException
    {
    	int type = te.getType();
    	TypeInfo retType = new TypeInfoImpl();
        
        
        if (type == TypeConstants.doubleType)
        {
        	retType.setType(TypeConstants.doubleType);
            
            return retType;
        }
        
        if (type == TypeConstants.floatType)
        {
    
        	retType.setType(TypeConstants.floatType);
            return retType;
        }
        
        if (type == TypeConstants.longType)
        {
        	retType.setType(TypeConstants.longType);
            
            return retType;
        }

      
   
        // short byte char get converted to integer
        retType.setType(TypeConstants.intType);
        return retType;
    }
    
    public static boolean validateActualParameterTypes(List formalParams,
            List passedParams) throws SPLException
    {
        if (formalParams.size() == passedParams.size())
        {
            for (int i = 0; i < formalParams.size(); i++)
            {
            	TypeInfo fParam = (TypeInfo) formalParams.get(i);
                Expression pParam = (Expression) passedParams.get(i);
                
                if (!TypeResolver.isTypeAssignableForEquality(
                        fParam, pParam.getType()))
                {
                   //System.out.println("type mismatch fParam.intValue()="+fParam+" pParam.getType()"+pParam.getType());
       
                    throw new IllegalParameterTypeException(Messages.getString(
							"SPL_METHOD_PASSED_ARGUMENTS_EXCEPTION_MSG",
							new Object[] { "" }));
                }
            }
        }
        return true;
    }
    
    public static boolean areTypesEqual(TypeInfo t1, TypeInfo t2)
    {
    	boolean result = false;
    	if(t1.getType() == t2.getType() && t1.getIsArray() == t2.getIsArray())
    	{
		
    		if(t1.getType() == TypeConstants.referenceType)
    		{
    			if(t1.getReferenceTypeName().equalsIgnoreCase(t2.getReferenceTypeName()))
    			{
    				result = true;
    			}
    			else{
    				result = false;
    			}
    		}
    		else
    		{
    			result = true;
    		}
    		
    	}
    	
    	
    	return result;
    }
    
    public static boolean isValidType(int tp)
    {
    
    	if (tp > 0 && tp < 14)
    	{
    		return true;
    	}
    	else
    	{
    		return false;
    	}
    }
    
}
