/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.  
 */
/**/
package org.apache.imperius.util;

import java.io.IOException;
import java.util.logging.FileHandler;
import java.util.logging.Level;
import java.util.logging.Logger;
import java.util.logging.SimpleFormatter;

public class SPLLogger 
{
	private static SPLLogger splLogger = null;
	private Logger auditLogger = null;
	private SPLLoggerClass logger = null;
	private static final String BUNDLE_NAME = "org.apache.imperius.util.messages"; //$NON-NLS-1$
	private static final String LOG_FILENAME = "SPL"; //$NON-NLS-1$
	private static final String AUDIT_LOG_FILENAME = "SPLAudit"; //$NON-NLS-1$
	private static boolean ENABLE_SPL_LOGGING = true;
	
	public static final String fileSeparator = System.getProperty("file.separator");
	
	/**
	 * Location of log files. 
	 * Initialized to the current directory (i.e. user.dir property)
	 */
	public static String SPL_HOME = System.getProperty("user.dir");
	
	// SPL Log Handler
	private FileHandler loggerHandler = null; 
	
	private SPLLogger()
	{
		initialize();
	}
	
	public static synchronized SPLLogger getSPLLogger()
	{
		if(splLogger == null)
		{
			splLogger = new SPLLogger();
		}
		return splLogger;
	}

	private void initialize()
	{
		try
		{
			// check if SPLHOME is set (Path to place log files)
			String userHome = System.getProperty("SPLHOME");
			if(userHome != null)
				SPL_HOME = userHome;
			
			String enableLogging = System.getProperty("ENABLE_SPL_LOGGING");
            if(enableLogging != null)
            {
            	if(enableLogging.equalsIgnoreCase("false"))
            	{
            		ENABLE_SPL_LOGGING = false;
            	}
            }

			// initialize SPL Audit Logger
			initSPLAuditLogger();
			
			// initialize SPL Logger
			initSPLLogger();
		}
		catch(SecurityException se)
		{
			System.err.println("Error! while initializing logger " + se.getLocalizedMessage());
		}
		catch(IOException io)
		{
			System.err.println("Error! while initializing logger " + io.getLocalizedMessage());
		}
	}
	
	public void initSPLAuditLogger() throws SecurityException, IOException
	{
		if(auditLogger != null)
			return;
		
		FileHandler auditHandler = null;

		// set auditLogger
		auditLogger = Logger.getLogger("AuditLog", BUNDLE_NAME);

		// create handler and add to logger
		auditHandler = new FileHandler(SPL_HOME
				+ fileSeparator + AUDIT_LOG_FILENAME
				+ ".log", true);
		auditLogger.addHandler(auditHandler);

		// initialize log formatter and set to handler
		SimpleFormatter auditFormatter = new SimpleFormatter();
		auditHandler.setFormatter(auditFormatter);
		
		auditLogger.setLevel(Level.ALL);
	}
	
	public void initSPLLogger() throws SecurityException, IOException
	{
		if(logger != null)
			return;
		
		logger = SPLLoggerClass.getInstance(SPLLogger.class.getName(), BUNDLE_NAME);
		
		// add handler to logger
		logger.addHandler(getSPLLogHandler()); 
	}

	public void enableSPLLogger()
	{
		if(!ENABLE_SPL_LOGGING)
		{
			logger.setLevel(Level.SEVERE);
		}
		else
		{
			logger.setLevel(Level.ALL);
		}
	}
	
	public synchronized SPLLoggerClass getLogger()
    {
        return logger;
    }
	
	public synchronized Logger getAuditLogger()
    {
        return auditLogger;
    }
	
	public void disableSPLLogger()
	{
		logger.setLevel(Level.OFF);
	}

	private FileHandler getSPLLogHandler() throws IOException
	{
		if(loggerHandler == null)
		{
			// create log handler
			loggerHandler = new FileHandler(SPL_HOME + fileSeparator
					+ LOG_FILENAME + ".log", false);

			logger.addHandler(loggerHandler);

			// initialize log formatter
			SimpleFormatter loggerFormatter = new SimpleFormatter();
			loggerHandler.setFormatter(loggerFormatter);
		}
		
		return loggerHandler;
	}
}
